# This is where the optimization is done.

import tkinter as tk
import numpy as np
import time, os, datetime
from tkinter import messagebox
from tkinter.filedialog import askopenfilenames
from multiprocessing import cpu_count

try:
    import any_files.main_application
    import any_files.optimize as op
    import any_files.example_data as test
except ModuleNotFoundError:
    import ANYstructure.any_files.main_application
    import ANYstructure.any_files.optimize as op
    import ANYstructure.any_files.example_data as test

class CreateOptimizeCylinderWindow():
    '''
    This class initiates the single optimization window.
    '''

    def __init__(self,master,app=None):
        super(CreateOptimizeCylinderWindow,self).__init__()
        if __name__ == '__main__':
            import pickle
            self._initial_structure_obj = test.get_structure_calc_object(heavy=True)
            self._initial_calc_obj = test.get_structure_calc_object(heavy=True)
            self._fatigue_object = test.get_fatigue_object()
            self._fatigue_pressure = test.get_fatigue_pressures()
            self._slamming_pressure = test.get_slamming_pressure()
            image_dir = os.path.dirname(__file__)+'\\images\\'
            self._PULS_object = None
            self._puls_acceptance = 0.87

            self._initial_cylinder_obj = calc.CylinderAndCurvedPlate(main_dict=test.shell_main_dict,
                                                                     shell=calc.Shell(test.shell_dict),
                                            long_stf=calc.Structure(test.obj_dict_cyl_long2),
                                            ring_stf=None,#calc.Structure(test.obj_dict_cyl_ring2),
                                            ring_frame=None)#calc.Structure(test.obj_dict_cyl_heavy_ring2))

            self._ML_buckling = dict()  # Buckling machine learning algorithm
            for name, file_base in zip(['cl SP buc int predictor', 'cl SP buc int scaler',
                                        'cl SP ult int predictor', 'cl SP ult int scaler',
                                        'cl SP buc GLGT predictor', 'cl SP buc GLGT scaler',
                                        'cl SP ult GLGT predictor', 'cl SP ult GLGT scaler',
                                        'cl UP buc int predictor', 'cl UP buc int scaler',
                                        'cl UP ult int predictor', 'cl UP ult int scaler',
                                        'cl UP buc GLGT predictor', 'cl UP buc GLGT scaler',
                                        'cl UP ult GLGT predictor', 'cl UP ult GLGT scaler'
                                        ],
                                       ["ml_files\\CL_output_cl_buc_predictor_In-plane_support_cl_1_SP",
                                        "ml_files\\CL_output_cl_buc_scaler_In-plane_support_cl_1_SP",
                                        "ml_files\\CL_output_cl_ult_predictor_In-plane_support_cl_1_SP",
                                        "ml_files\\CL_output_cl_ult_scaler_In-plane_support_cl_1_SP",
                                        "ml_files\\CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_SP",
                                        "ml_files\\CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_SP",
                                        "ml_files\\CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_SP",
                                        "ml_files\\CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_SP",
                                        "ml_files\\CL_output_cl_buc_predictor_In-plane_support_cl_1_UP",
                                        "ml_files\\CL_output_cl_buc_scaler_In-plane_support_cl_1_UP",
                                        "ml_files\\CL_output_cl_ult_predictor_In-plane_support_cl_1_UP",
                                        "ml_files\\CL_output_cl_ult_scaler_In-plane_support_cl_1_UP",
                                        "ml_files\\CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_UP",
                                        "ml_files\\CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_UP",
                                        "ml_files\\CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_UP",
                                        "ml_files\\CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_UP",
                                        "ml_files\\CL_CSR-Tank_req_cl_predictor",
                                        "ml_files\\CL_CSR-Tank_req_cl_UP_scaler",
                                        "ml_files\\CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_predictor",
                                        "ml_files\\CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_SP_scaler"]):
                self._ML_buckling[name] = None
                if os.path.isfile(file_base + '.pickle'):
                    file = open(file_base + '.pickle', 'rb')
                    self._ML_buckling[name] = pickle.load(file)
                    file.close()

            self._ML_classes = {0: 'N/A',
                                1: 'A negative utilisation factor is found.',
                                2: 'At least one of the in-plane loads must be non-zero.',
                                3: 'Division by zero',
                                4: 'Overflow',
                                5: 'The aspect ratio exceeds the PULS code limit',
                                6: 'The global slenderness exceeds 4. Please reduce stiffener span or increase stiffener height.',
                                7: 'The applied pressure is too high for this plate field.', 8: 'web-flange-ratio',
                                9: 'UF below or equal 0.87', 10: 'UF between 0.87 and 1.0', 11: 'UF above 1.0'}
        else:
            self.app = app
            self._initial_structure_obj = app._line_to_struc[app._active_line][0]
            self._initial_calc_obj = app._line_to_struc[app._active_line][1]
            self._initial_cylinder_obj = app._line_to_struc[app._active_line][5]
            self._fatigue_object = app._line_to_struc[app._active_line][2]
            try:
                self._fatigue_pressure = app.get_fatigue_pressures(app._active_line,
                                                                   self._fatigue_object.get_accelerations())
            except AttributeError:
                self._fatigue_pressure = None
            try:
                self._lateral_pressure = 0
            except KeyError:
                self._lateral_pressure = 0
            try:

                if self.app.get_highest_pressure(self.app._active_line)['slamming'] is None:
                    self._slamming_pressure = 0
                else:
                    self._slamming_pressure = self.app.get_highest_pressure(self.app._active_line)['slamming']
            except KeyError:
                self._slamming_pressure = 0
            image_dir = app._root_dir +'\\images\\'
            self._root_dir = app._root_dir
            self._PULS_object = app._PULS_results
            self._puls_acceptance = self.app._new_puls_uf.get()

            self._ML_buckling = app._ML_buckling



        self._frame = master
        self._frame.wm_title("Optimize structure")
        self._frame.geometry('1600x900')
        self._frame.grab_set()

        '''
            shell_upper_bounds = np.array( [0.03, 3, 5, 5, 10, None, None, None])
            shell_deltas = np.array(       [0.005, 0.5, 1, 0.1,1, None, None, None])
            shell_lower_bounds = np.array( [0.02, 2.5, 5, 5, 10, None, None, None])
        
            long_upper_bounds = np.array(   [0.8, None, 0.5, 0.02, 0.2, 0.03, None, None])
            long_deltas = np.array(         [0.1, None, 0.1, 0.01, 0.1, 0.01, None, None])
            long_lower_bounds = np.array(   [0.7, None, 0.3,  0.01, 0.1, 0.01, None, None])
        
            ring_stf_upper_bounds = np.array(   [None, None, 0.5, 0.018, 0.2, 0.03, None, None])
            ring_stf_deltas = np.array(         [None, None, 0.1, 0.004, 0.1, 0.01, None, None])
            ring_stf_lower_bounds = np.array(   [None, None, 0.3,  0.010, 0.1, 0.010, None, None])
        
            ring_frame_upper_bounds = np.array( [None, None, 0.9, 0.04, 0.3, 0.04, None, None])
            ring_frame_deltas = np.array(       [None, None, 0.2, 0.01, 0.1, 0.01, None, None])
            ring_frame_lower_bounds = np.array( [None, None, 0.7,  0.02, 0.2, 0.02, None, None])
        '''
        ent_w = 12

        default_shell_upper_bounds = np.array([0.03, 3, 5, 5, 10, None, None, None])
        default_shell_deltas = np.array([0.005, 0.5, 1, 0.1, 1, None, None, None])
        default_shell_lower_bounds = np.array([0.02, 2.5, 5, 5, 10, None, None, None])

        default_long_upper_bounds = np.array([0.8, None, 0.5, 0.02, 0.2, 0.03, None, None])
        default_long_deltas = np.array([0.1, None, 0.1, 0.01, 0.1, 0.01, None, None])
        default_long_lower_bounds = np.array([0.7, None, 0.3, 0.01, 0.1, 0.01, None, None])

        default_ring_stf_upper_bounds = np.array([None, None, 0.5, 0.018, 0.2, 0.03, None, None])
        default_ring_stf_deltas = np.array([None, None, 0.1, 0.004, 0.1, 0.01, None, None])
        default_ring_stf_lower_bounds = np.array([None, None, 0.3, 0.010, 0.1, 0.010, None, None])

        default_ring_frame_upper_bounds = np.array([None, None, 0.9, 0.04, 0.3, 0.04, None, None])
        default_ring_frame_deltas = np.array([None, None, 0.2, 0.01, 0.1, 0.01, None, None])
        default_ring_frame_lower_bounds = np.array([None, None, 0.7, 0.02, 0.2, 0.02, None, None])
        
        self._default_data =  [[default_shell_upper_bounds,default_shell_deltas, default_shell_lower_bounds],
                              [default_long_upper_bounds, default_long_deltas, default_long_lower_bounds],
                              [default_ring_stf_upper_bounds, default_ring_stf_deltas, default_ring_stf_lower_bounds],
                              [default_ring_frame_upper_bounds, default_ring_frame_deltas,
                               default_ring_frame_lower_bounds]]

        shell_example = [0.03, 3, 5, 5, 10, None, None, None]
        long_example = ring_stf_example = ring_frame_example =  [0.8, None, 0.5, 0.02, 0.2, 0.03, None, None]

        shell_upper_bounds = [tk.DoubleVar() for dummy in shell_example]
        shell_deltas = [tk.DoubleVar() for dummy in shell_example]
        shell_lower_bounds = [tk.DoubleVar() for dummy in shell_example]
        
        long_upper_bounds = [tk.DoubleVar() for dummy in long_example]
        long_deltas = [tk.DoubleVar() for dummy in long_example]
        long_lower_bounds = [tk.DoubleVar() for dummy in long_example]
        
        ring_stf_upper_bounds = [tk.DoubleVar() for dummy in ring_stf_example]
        ring_stf_deltas = [tk.DoubleVar() for dummy in ring_stf_example]
        ring_stf_lower_bounds = [tk.DoubleVar() for dummy in ring_stf_example]
        
        ring_frame_upper_bounds = [tk.DoubleVar() for dummy in ring_frame_example]
        ring_frame_deltas = [tk.DoubleVar() for dummy in ring_frame_example]
        ring_frame_lower_bounds = [tk.DoubleVar() for dummy in ring_frame_example]
        self._new_geo_data = list()

        self._new_geo_data =  [[shell_upper_bounds,shell_deltas, shell_lower_bounds],
                              [long_upper_bounds, long_deltas, long_lower_bounds],
                              [ring_stf_upper_bounds, ring_stf_deltas, ring_stf_lower_bounds],
                              [ring_frame_upper_bounds, ring_frame_deltas, ring_frame_lower_bounds]]

        self._new_entries = list()
        map_type = {'shell':0, 'long':1, 'ring stf': 2, 'ring heavy':3}
        map_type_idx = {0: True, 1: self._initial_cylinder_obj.LongStfObj is not None,
                        2: self._initial_cylinder_obj.RingStfObj is not None,
                        3: self._initial_cylinder_obj.RingFrameObj is not None}

        for idx_1, geo_i in enumerate(self._new_geo_data):
            all_geos = list()
            if map_type_idx[idx_1] == False:
                continue
            for idx_2, entries in enumerate(geo_i):
                these_ents = list()
                for idx_3, ent_i in enumerate(entries):
                    self._new_geo_data[idx_1][idx_2][idx_3].trace('w', self.update_running_time)
                    these_ents.append(tk.Entry(self._frame,
                                               textvariable = self._new_geo_data[idx_1][idx_2][idx_3], width = ent_w))
                    self._new_geo_data[idx_1][idx_2][idx_3].set(0 if self._default_data[idx_1][idx_2][idx_3] is None
                                                                else self._default_data[idx_1][idx_2][idx_3]*1000)
                all_geos.append(these_ents)
            self._new_entries.append(all_geos)
        
        self._predefined_stiffener_iter = None

        self._opt_runned = False
        self._opt_results = ()
        self._opt_actual_running_time = tk.Label(self._frame,text='',font='Verdana 12 bold')


        self._draw_scale = 600
        self._canvas_dim = (550, 500)
        self._canvas_opt = tk.Canvas(self._frame,width=self._canvas_dim[0], height=self._canvas_dim[1], 
                                     background='azure',relief = 'groove', borderwidth=2)


        # tk.Frame(self._frame,width=770,height=5, bg="grey", colormap="new").place(x=20,y=127)
        # tk.Frame(self._frame, width=770, height=5, bg="grey", colormap="new").place(x=20, y=167)

        self._canvas_opt.place(x=1000,y=350)

        algorithms = ('anysmart cylinder','random','random_no_delta')

        tk.Label(self._frame,text='-- Cylinder optimizer --',font='Verdana 15 bold').place(x=10,y=10)


        # upper and lower bounds for optimization
        #[0.6, 0.012, 0.3, 0.01, 0.1, 0.01]

        self._new_algorithm = tk.StringVar()
        self._new_algorithm_random_trials = tk.IntVar()
        self._new_swarm_size = tk.IntVar()
        self._new_omega  = tk.DoubleVar()
        self._new_phip  = tk.DoubleVar()
        self._new_phig  = tk.DoubleVar()
        self._new_maxiter = tk.IntVar()
        self._new_minstep  = tk.DoubleVar()
        self._new_minfunc  = tk.DoubleVar()
        self._new_slamming_pressure = tk.DoubleVar()
        self._new_fatigue_int_press = tk.DoubleVar()
        self._new_fatigue_ext_press = tk.DoubleVar()

        #additional choices for the random and pso algorithm
        self._ent_algorithm = tk.OptionMenu(self._frame,self._new_algorithm,command=self.selected_algorithm,*algorithms)
        self._ent_random_trials = tk.Entry(self._frame,textvariable=self._new_algorithm_random_trials)

        pso_width = 10
        self._ent_swarm_size = tk.Entry(self._frame,textvariable=self._new_swarm_size, width = pso_width)
        self._ent_omega = tk.Entry(self._frame,textvariable=self._new_omega, width = pso_width)
        self._ent_phip = tk.Entry(self._frame,textvariable=self._new_phip, width = pso_width)
        self._ent_phig = tk.Entry(self._frame,textvariable=self._new_phig, width = pso_width)
        self._ent_maxiter = tk.Entry(self._frame,textvariable=self._new_maxiter, width = pso_width)
        self._ent_minstep = tk.Entry(self._frame,textvariable=self._new_minstep, width = pso_width)
        self._ent_minfunc = tk.Entry(self._frame,textvariable=self._new_minfunc, width = pso_width)



        # stresses in plate and stiffener

        self._new_sasd = tk.DoubleVar()
        self._new_smsd = tk.DoubleVar()
        self._new_tTsd = tk.DoubleVar()
        self._new_tQsd = tk.DoubleVar()
        self._new_design_pressure = tk.DoubleVar()
        self._new_shsd = tk.DoubleVar()
        
        self._new_sasd.set(self._initial_cylinder_obj.sasd)
        self._new_smsd.set(self._initial_cylinder_obj.smsd)
        self._new_tTsd.set(self._initial_cylinder_obj.tTsd)
        self._new_tQsd.set(self._initial_cylinder_obj.tQsd)
        self._new_design_pressure.set(self._initial_cylinder_obj.psd)
        self._new_shsd.set(self._initial_cylinder_obj.shsd)


        self._ent_sasd = tk.Entry(self._frame, textvariable=self._new_sasd, width=ent_w)
        self._ent_smsd = tk.Entry(self._frame, textvariable=self._new_smsd, width=ent_w)
        self._ent_tTsd = tk.Entry(self._frame, textvariable=self._new_tTsd, width=ent_w)
        self._ent_tQsd = tk.Entry(self._frame, textvariable=self._new_tQsd, width=ent_w)
        self._ent_design_pressure = tk.Entry(self._frame, textvariable=self._new_design_pressure, width=ent_w)
        self._ent_shsd = tk.Entry(self._frame, textvariable=self._new_shsd, width=ent_w)


        start_x,start_y,dx,dy = 20,100,100,40

        self._new_processes = tk.IntVar()
        self._new_processes.set(max(cpu_count() - 1, 1))
        tk.Label(self._frame, text='Processes\n (CPUs)', font='Verdana 9 bold', bg = 'silver')\
            .place(x=start_x + 10 * dx, y=start_y - 1.1 * dy)
        tk.Entry(self._frame, textvariable=self._new_processes, width = 12, bg = 'silver')\
            .place(x=start_x + 10 * dx, y=start_y - 0.3 * dy)




        self._runnig_time_label = tk.Label(self._frame, text='',font='Verdana 12 bold', fg = 'red')
        self._runnig_time_label.place(x=start_x+4.3*dx, y=start_y + 2.8 * dy)
        #tk.Label(self._frame, text='seconds ',font='Verdana 9 bold').place(x=start_x+6*dx, y=start_y + 2.8 * dy)
        self._result_label = tk.Label(self._frame, text = '',font = 'Verdana 9 bold' )
        self._result_label.place(x=start_x, y=start_y + 4.2 * dy)

        '''
                self._new_geo_data =  [[shell_upper_bounds,shell_deltas, shell_lower_bounds],
                              [long_upper_bounds, long_deltas, long_lower_bounds],
                              [ring_stf_upper_bounds, ring_stf_deltas, ring_stf_lower_bounds],
                              [ring_frame_upper_bounds, ring_frame_deltas, ring_frame_lower_bounds]]
        '''
        shell = ['Shell thk. [mm]', 'Shell radius [mm]', 'l rings [mm]', 'L shell [mm]', 'L tot. [mm]', 'N/A - future',
                 'N/A - future', 'N/A - future']
        stf_long = ['Spacing [mm]', 'N/A', 'Web height [mm]', 'Web thk. [mm]', 'Flange width [mm]',
                    'Flange thk. [mm]', 'N/A - future', 'N/A - future']
        stf_ring = ['N/A', 'N/A', 'Web height [mm]', 'Web thk. [mm]', 'Flange width [mm]',
                    'Flange thk. [mm]', 'N/A - future', 'N/A - future']
        all_label = [shell, stf_long, stf_ring, stf_ring]
        text_i = ['Upper bounds [mm]', 'Iteration delta [mm]','Lower bounds [mm]']
        kind = ['Shell or panel', 'Longitudinal stiffener', 'Ring stiffener', 'Ring frame/girder']
        for idx_1, member in enumerate(self._new_entries):
            if map_type_idx[idx_1] == False:
                continue
            for idx_2, bounds in enumerate(member):
                tk.Label(self._frame, text=text_i[idx_2], font='Verdana 9').place(x=start_x,
                                                                                  y=start_y + dy * idx_1 * 4 + dy * idx_2)
                if idx_2 == 0:
                    tk.Label(self._frame, text=kind[idx_1], font='Verdana 10 bold') \
                        .place(x=start_x, y=start_y + dy * idx_1 * 4 + dy * idx_2 - dy * 0.5)
                for idx_3, entry_i in enumerate(bounds):
                    if idx_2 == 0:
                        tk.Label(self._frame, text=all_label[idx_1][idx_3], font='Verdana 7 bold')\
                            .place(x = start_x+dx*2 + idx_3*dx, y = start_y+dy*idx_1*4 + dy*idx_2 -dy*0.5)

                    entry_i.place(x = start_x+dx*2 + idx_3*dx, y = start_y+dy*idx_1*4 + dy*idx_2)
                    if 'N/A' in all_label[idx_1][idx_3]:
                        entry_i.configure(bg = 'grey')


        
        ###
        
        #Labels for the pso
 
        self._lb_swarm_size = tk.Label(self._frame,text='swarm size')
        self._lb_omega  = tk.Label(self._frame,text='omega')
        self._lb_phip  = tk.Label(self._frame,text='phip')
        self._lb_phig  = tk.Label(self._frame,text='phig')
        self._lb_maxiter = tk.Label(self._frame,text='maxiter')
        self._lb_minstep  = tk.Label(self._frame,text='minstep')
        self._lb_minfunc  = tk.Label(self._frame,text='minfunc')

        ###
        dys = 0.9*dy
        tk.Label(self._frame, text='Design axial stress,          sa,sd', font='Verdana 9')\
            .place(x=start_x+10*dx,y=start_y+1*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+1*dys)

        tk.Label(self._frame, text='Design bending stress,   sm,sd', font='Verdana 9')\
            .place(x=start_x+10*dx,y=start_y+2*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+2*dys)

        tk.Label(self._frame, text='Design torsional stress,   tT,sd', font='Verdana 9')\
            .place(x=start_x+10*dx,y=start_y+3*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+3*dys)

        tk.Label(self._frame, text='Design shear stress,        tQ,sd', font='Verdana 9')\
            .place(x=start_x+10*dx,y=start_y+4*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+4*dys)

        tk.Label(self._frame, text='Design lateral pressure,    psd', font='Verdana 9 bold')\
            .place(x=start_x+10*dx,y=start_y+5*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+5*dys)
        
        tk.Label(self._frame, text='Additional hoop stress, sh,sd,    psd', font='Verdana 9 bold')\
            .place(x=start_x+10*dx,y=start_y+6*dys)
        tk.Label(self._frame, text='Pa', font='Verdana 9')\
            .place(x=start_x+dx*14,y=start_y+6*dys)
        
        self._ent_sasd.place(x=start_x+dx*13,y=start_y+1*dys)
        self._ent_smsd.place(x=start_x+dx*13,y=start_y+2*dys)
        self._ent_tTsd.place(x=start_x+dx*13,y=start_y+3*dys)
        self._ent_tQsd.place(x=start_x + dx * 13, y=start_y + 4 * dys)
        self._ent_design_pressure.place(x=start_x + dx * 13, y=start_y + 5 * dys)
        self._ent_shsd.place(x=start_x + dx * 13, y=start_y +6 * dys)
        

        if self._fatigue_pressure is not None:
            tk.Label(self._frame, text='Fatigue pressure: internal= '+str(self._fatigue_pressure['p_int'])+ ' external= '
                                       +str(self._fatigue_pressure['p_ext']), font='Verdana 7') \
                .place(x=start_x + dx * 5, y=start_y + 19.3 * dy)
        else:
            tk.Label(self._frame, text='Fatigue pressure: internal= '+str(0)+ ' external= '
                                       +str(0), font='Verdana 7') \
                .place(x=start_x + dx * 5, y=start_y + 19.3 * dy)




        #setting default values
        init_dim = float(10) #mm
        init_thk = float(1) #mm

        self._new_slamming_pressure.set(self._slamming_pressure)
        if self._fatigue_pressure is None:
            self._new_fatigue_ext_press.set(0), self._new_fatigue_int_press.set(0)
        else:
            self._new_fatigue_int_press.set(self._fatigue_pressure['p_int']), \
            self._new_fatigue_ext_press.set(self._fatigue_pressure['p_ext'])

        self._new_algorithm.set('anysmart cylinder')
        self._new_algorithm_random_trials.set(100000)
        self._new_swarm_size.set(100)
        self._new_omega.set(0.5)
        self._new_phip.set(0.5)
        self._new_phig.set(0.5)
        self._new_maxiter.set(100)
        self._new_minstep.set(1e-8)
        self._new_minfunc.set(1e-8)

        self._new_algorithm_random_trials.trace('w',self.update_running_time)
        self._new_algorithm.trace('w',self.update_running_time)


        # self.running_time_per_item = {'PULS':0.2489626556016598, 'RP': 1.009943181818182e-5}
        # self.initial_weight = op.calc_weight([self._spacing,self._pl_thk,self._stf_web_h,self._stf_web_thk,
        #                                       self._fl_w,self._fl_thk,self._new_span.get(),self._new_width_lg.get()])

        # img_file_name = 'img_plate_and_stiffener.gif'
        # if os.path.isfile('images/' + img_file_name):
        #     file_path = 'images/' + img_file_name
        # else:
        #     file_path = self._root_dir + '/images/' + img_file_name
        # photo = tk.PhotoImage(file=file_path)
        # label = tk.Label(self._frame,image=photo)
        # label.image = photo  # keep a reference!
        # label.place(x=550, y=300)


        # tk.Label(self._frame,text='Select algorithm', font = 'Verdana 8 bold').place(x=start_x+dx*11, y=start_y+7*dy)
        # self._ent_algorithm.place(x=start_x+dx*11, y=start_y+dy*8)
        self.algorithm_random_label = tk.Label(self._frame, text='Number of trials')

        # tk.Button(self._frame,text='algorith information',command=self.algorithm_info,bg='white')\
        #     .place(x=start_x+dx*12.5, y=start_y+dy*7)
        self.run_button = tk.Button(self._frame,text='RUN OPTIMIZATION!', command=self.run_optimizaion, bg='red',
                                    font='Verdana 10 bold',fg='Yellow', relief="raised")
        self.run_button.place(x=start_x+dx*11.5, y=start_y-dy, relwidth = 0.15)
        # self.run_results = tk.Button(self._frame,text='show calculated', command=self.plot_results, bg='white',
        #                             font='Verdana 10',fg='black')
        # self.run_results.place(x=start_x+dx*8, y=start_y+dy*1.5)
        self._opt_actual_running_time.place(x=start_x+dx*11, y=start_y)

        self.close_and_save =tk.Button(self._frame,text='Return and replace initial structure with optimized',
                                       command=self.save_and_close,bg='green',font='Verdana 10',fg='yellow')
        self.close_and_save.place(x=start_x+dx*5,y=10)

        tk.Button(self._frame, text='Open predefined stiffeners example',
                  command=self.open_example_file, bg='white', font='Verdana 10')\
            .place(x=start_x+dx*10,y=10)


        # Stress scaling
        self._new_fup = tk.DoubleVar()
        self._new_fup.set(0.5)
        self._new_fdwn = tk.DoubleVar()
        self._new_fdwn.set(1)

        tk.Label(self._frame, text='Factor when scaling stresses up, fup')\
            .place(x=start_x, y=start_y + 16 * dy)
        ent_fup = tk.Entry(self._frame, textvariable=self._new_fup, width = 10)
        ent_fup.place(x=start_x + dx * 3, y=start_y + 16 * dy)
        tk.Label(self._frame, text='Factor when scaling stresses up, fdown')\
            .place(x=start_x, y=start_y + 17 * dy)
        ent_fdwn = tk.Entry(self._frame, textvariable=self._new_fdwn, width = 10)
        ent_fdwn.place(x=start_x + dx * 3, y=start_y + 17 * dy)

        # tk.Button(self._frame,text='Iterate predefiened stiffeners',command=self.open_multiple_files ,bg='yellow')\
        #     .place(x=start_x, y=start_y - dy * 2)
        # command=lambda id="default": self.set_colors(id)
        self._toggle_btn = tk.Button(self._frame, text="Iterate predefiened stiffeners", relief="raised",
                                     command=self.toggle, bg = 'salmon')
        self._toggle_btn.place(x= + 3*dx, y=start_y - dy * 2)
        self._toggle_object, self._filez = self._initial_structure_obj, None
        self.draw_properties()
        self.update_running_time()

        main_application.Application.draw_cylinder(text_size='Verdana 8 bold',
                                                   canvas = self._canvas_opt,
                                                   CylObj=self._initial_cylinder_obj,
                                                   start_x_cyl=350, start_y_cyl=300, text_x=230,
                                                   text_y=120)

    def selected_algorithm(self,event):
        '''
        Action when selecting an algorithm.
        :return:
        '''
        start_x, start_y, dx, dy = 20, 100, 100, 40
        if self._new_algorithm.get()=='random' or self._new_algorithm.get()=='random_no_delta':
            self._ent_random_trials.place_forget()
            self.algorithm_random_label.place_forget()
            self._lb_swarm_size.place_forget()
            self._lb_omega.place_forget()
            self._lb_phip.place_forget()
            self._lb_phig.place_forget()
            self._lb_maxiter.place_forget()
            self._lb_minstep.place_forget()
            self._lb_minfunc.place_forget()
            self._ent_swarm_size.place_forget()
            self._ent_omega.place_forget()
            self._ent_phip.place_forget()
            self._ent_phig.place_forget()
            self._ent_maxiter.place_forget()
            self._ent_minstep.place_forget()
            self._ent_minfunc.place_forget()
            self._ent_random_trials.place(x=start_x+dx*11.3, y=start_y+1.2*dy)
            self.algorithm_random_label.place(x=start_x+dx*11.3, y=start_y+0.5*dy)
        elif self._new_algorithm.get()=='anysmart' or self._new_algorithm.get()=='anydetail':
            self._ent_random_trials.place_forget()
            self.algorithm_random_label.place_forget()
            self._lb_swarm_size.place_forget()
            self._lb_omega.place_forget()
            self._lb_phip.place_forget()
            self._lb_phig.place_forget()
            self._lb_maxiter.place_forget()
            self._lb_minstep.place_forget()
            self._lb_minfunc.place_forget()
            self._ent_swarm_size.place_forget()
            self._ent_omega.place_forget()
            self._ent_phip.place_forget()
            self._ent_phig.place_forget()
            self._ent_maxiter.place_forget()
            self._ent_minstep.place_forget()
            self._ent_minfunc.place_forget()

        elif self._new_algorithm.get()=='pso':
            y_place_label =11.2
            y_place = 12.2
            self._ent_random_trials.place_forget()
            
            self._lb_swarm_size.place(x=start_x+dx*y_place_label, y=start_y-2*dy)
            self._lb_omega.place(x=start_x+dx*y_place_label, y=start_y-1*dy)
            self._lb_phip.place(x=start_x+dx*y_place_label, y=start_y-0*dy)
            self._lb_phig.place(x=start_x+dx*y_place_label, y=start_y+1*dy)
            self._lb_maxiter.place(x=start_x+dx*y_place_label, y=start_y+2*dy)
            self._lb_minstep.place(x=start_x+dx*y_place_label, y=start_y+3*dy)
            self._lb_minfunc.place(x=start_x+dx*y_place_label, y=start_y+4*dy)
            
            self._ent_swarm_size.place(x=start_x+dx*y_place, y=start_y-2*dy)
            self._ent_omega.place(x=start_x+dx*y_place, y=start_y-1*dy)
            self._ent_phip.place(x=start_x+dx*y_place, y=start_y+0*dy)
            self._ent_phig.place(x=start_x+dx*y_place, y=start_y+1*dy)
            self._ent_maxiter.place(x=start_x+dx*y_place, y=start_y+2*dy)
            self._ent_minstep.place(x=start_x+dx*y_place, y=start_y+3*dy)
            self._ent_minfunc.place(x=start_x+dx*y_place, y=start_y+4*dy)

    def modify_structure_object(self):
        ''' Chaning parameters in the structure object before running. '''
        pass

    def run_optimizaion(self):
        '''
        function for button
        :return:
        '''

        self.run_button.config(bg = 'white')
        self.run_button.config(fg='red')
        self.run_button.config(text='RUNNING OPTIMIZATION')
        self.run_button.config(relief="sunken")
        self._opt_actual_running_time.config(text='Run started ' + datetime.datetime.now().strftime("%H:%M:%S"))
        self._opt_actual_running_time.update()
        t_start = time.time()
        self._opt_results, self._opt_runned = (), False

        self.pso_parameters = (self._new_swarm_size.get(),self._new_omega.get(),self._new_phip.get(),
                               self._new_phig.get(),
                               self._new_maxiter.get(),self._new_minstep.get(),self._new_minfunc.get())


        if self._fatigue_pressure is not None:

            fat_press = ((self._fatigue_pressure['p_ext']['loaded'],self._fatigue_pressure['p_ext']['ballast'],
                          self._fatigue_pressure['p_ext']['part']),
                         (self._fatigue_pressure['p_int']['loaded'],self._fatigue_pressure['p_int']['ballast'],
                          self._fatigue_pressure['p_int']['part']))
        else:
            fat_press = None

        self._new_sasd.set(self._new_sasd.get())
        self._new_smsd.set(self._new_smsd.get())
        self._new_tTsd.set(self._new_tTsd.get())
        self._new_tQsd.set(self._new_tQsd.get())
        self._new_design_pressure.set(self._new_design_pressure.get())
        self._initial_cylinder_obj.psd = self._new_design_pressure.get()
        self._new_shsd.set(self._new_shsd.get())

        self._opt_results= op.run_optmizataion(initial_structure_obj= self._initial_cylinder_obj,
                                               min_var= self.get_lower_bounds(),
                                               max_var=self.get_upper_bounds(),lateral_pressure=
                                               self._new_design_pressure.get(),
                                               deltas= self.get_deltas(),algorithm=self._new_algorithm.get(),
                                               trials=self._new_algorithm_random_trials.get(),
                                               fatigue_obj=self._fatigue_object,
                                               fat_press_ext_int=fat_press,
                                               slamming_press = self._new_slamming_pressure.get(),
                                               predefined_stiffener_iter=self._predefined_stiffener_iter,
                                               processes=self._new_processes.get(),
                                               use_weight_filter = True,
                                               fdwn = self._new_fdwn.get(), fup = self._new_fdwn.get(),
                                               cylinder = True)

        if self._opt_results is not None and self._opt_results[0] is not None:
            self._opt_actual_running_time.config(text='Actual running time: \n'
                                                     +str(round((time.time()-t_start)/60,4))+' min')
            self._opt_actual_running_time.update()
            self._opt_runned = True
            #self._result_label.config(text=self._opt_results[0].__str__)
            self._canvas_opt.delete('all')

            main_application.Application.draw_cylinder(text_size='Verdana 8 bold',
                                                       canvas = self._canvas_opt,
                                                       CylObj=self._opt_results[0],
                                                       start_x_cyl=350, start_y_cyl=300, text_x=230,
                                                       text_y=120)
            self._new_sasd.set(self._opt_results[0].sasd)
            self._new_smsd.set(self._opt_results[0].smsd)
            self._new_tTsd.set(self._opt_results[0].tTsd)
            self._new_tQsd.set(self._opt_results[0].tQsd)
            self._new_design_pressure.set(self._opt_results[0].psd)
            self._new_shsd.set(self._opt_results[0].shsd)
            #self.draw_properties()
        else:
            messagebox.showinfo(title='Nothing found', message='No better alternatives found. Modify input.\n'
                                                               'There may be no alternative that is acceptable.\n')

        self.run_button.config(bg='green')
        self.run_button.config(fg='yellow')
        self.run_button.config(text='RUN OPTIMIZATION')
        self.run_button.config(relief="raised")

    def get_running_time(self):
        '''
        Estimate the running time of the algorithm.
        :return:
        '''

        pass

    def get_deltas(self):
        '''
        Return a numpy array of the deltas.
        :return:
        '''
        all_deltas = list()
        for idx_1, geo_i in enumerate(self._new_geo_data):
            these_deltas = list()
            for idx_3, val in enumerate(geo_i[1]):
                these_deltas.append(val.get()/1000)
            all_deltas.append(these_deltas)
        return all_deltas

    def update_running_time(self,*args):
        '''
        Estimate the running time of the algorithm.
        :return:
        '''

        pass
    
    def get_upper_bounds(self):
        '''
        Return an numpy array of upper bounds.
        :return: 
        '''
        all_upper = list()
        for idx_1, geo_i in enumerate(self._new_geo_data):
            these_upper = list()
            for idx_3, val in enumerate(geo_i[0]):
                these_upper.append(val.get()/1000)
            all_upper.append(these_upper)
        return all_upper
        
    def get_lower_bounds(self):
        '''
        Return an numpy array of lower bounds.
        :return: 
        '''
        all_lower = list()
        for idx_1, geo_i in enumerate(self._new_geo_data):
            these_lower = list()
            for idx_3, val in enumerate(geo_i[2]):
                these_lower.append(val.get()/1000)
            all_lower.append(these_lower)
        return all_lower

    def get_sigmas(self):
        '''
        Returns the stressess.
        :return:
        '''
        return np.array([self._new_sasd.get(),self._new_smsd.get(),
                         self._new_tTsd.get(),self._new_tQsd.get(),
                         self._new_design_pressure.get(),self._new_shsd.get()])

    def checkered(self,line_distance):
        # vertical lines at an interval of "line_distance" pixel
        for x in range(line_distance, self._canvas_dim[0], line_distance):
            self._canvas_opt.create_line(x, 0, x, self._canvas_dim[0], fill="grey",stipple='gray50')
        # horizontal lines at an interval of "line_distance" pixel
        for y in range(line_distance, self._canvas_dim[1], line_distance):
            self._canvas_opt.create_line(0, y, self._canvas_dim[0], y, fill="grey",stipple='gray50')

    def draw_properties(self):
        '''
        Drawing properties in the canvas.
        :return:
        '''
        self._canvas_opt.delete('all')
        #self.checkered(10)
        ctr_x = self._canvas_dim[0]/2
        ctr_y = self._canvas_dim[1]/2+200
        m = self._draw_scale
        init_color,init_stipple = 'blue','gray12'
        opt_color,opt_stippe = 'red','gray12'
        # self._canvas_opt.create_rectangle(0,0,self._canvas_dim[0]+10,80,fill='white')
        # self._canvas_opt.create_line(10,10,30,10,fill = init_color,width=5)


        if self._opt_runned:

            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_s() / 2, ctr_y,
                                             ctr_x + m * self._opt_results[0].get_s()  / 2,
                                             ctr_y - m * self._opt_results[0].get_pl_thk(), fill=opt_color,
                                             stipple=opt_stippe)

            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_web_thk() / 2, ctr_y -
                                             m * self._opt_results[0].get_pl_thk(),
                                             ctr_x + m * self._opt_results[0].get_web_thk() / 2,
                                             ctr_y - m * (self._opt_results[0].get_web_h() + self._opt_results[0].get_pl_thk())
                                             , fill=opt_color, stipple=opt_stippe)
            if self._opt_results[0].get_stiffener_type() not in ['L', 'L-bulb']:
                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_fl_w() / 2, ctr_y
                                                 - m * (self._opt_results[0].get_pl_thk()+ self._opt_results[0].get_web_h()),
                                                 ctr_x + m * self._opt_results[0].get_fl_w() / 2,ctr_y -
                                                 m * (self._opt_results[0].get_pl_thk() + self._opt_results[0].get_web_h() +
                                                      self._opt_results[0].get_fl_thk()),
                                                 fill=opt_color, stipple=opt_stippe)
            else:
                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_web_thk() / 2, ctr_y
                                                 - m * (self._opt_results[0].get_pl_thk()+ self._opt_results[0].get_web_h()),
                                                 ctr_x + m * self._opt_results[0].get_fl_w() ,ctr_y -
                                                 m * (self._opt_results[0].get_pl_thk() + self._opt_results[0].get_web_h() +
                                                      self._opt_results[0].get_fl_thk()),
                                                 fill=opt_color, stipple=opt_stippe)

            self._canvas_opt.create_line(10, 50, 30, 50, fill=opt_color, width=5)
            self._canvas_opt.create_text(270,50,text='Optimized - Pl.: '+str(round(self._opt_results[0].get_s()*1000,1))
                                                     +'x'+ str(round(self._opt_results[0].get_pl_thk()*1000,1))+
                                                     ' Stf.: '+str(round(self._opt_results[0].get_web_h()*1000,1))+
                                                    'x'+str(round(self._opt_results[0].get_web_thk()*1000,1))+'+'+
                                                            str(round(self._opt_results[0].get_fl_w()*1000,1))+
                                                    'x'+str(round(self._opt_results[0].get_fl_thk()*1000,1)),
                                        font = 'Verdana 8',fill = opt_color)

    def save_and_close(self):
        '''
        Save and close
        :return:
        '''

        if __name__ == '__main__':
            self._frame.destroy()
            return

        try:
            self.app.on_close_opt_cyl_window(self._opt_results)
        except (IndexError, TypeError):
            messagebox.showinfo(title='Nothing to return',message='No results to return.')
            return
        self._frame.destroy()

    def algorithm_info(self):
        ''' When button is clicked, info is displayed.'''

        messagebox.showinfo(title='Algorith information',
                            message='The algorithms currently included is:\n'
                                    'ANYSMART:  \n'
                                    '           Calculates all alternatives using upper and lower bounds.\n'
                                    '           The step used inside the bounds is defined in deltas.\n'
                                    '           This algoritm uses MULTIPROCESSING and will be faster.\n\n'
                                    'RANDOM:    \n'
                                    '           Uses the same bounds and deltas as in ANYSMART.\n'
                                    '           Number of combinations calculated is defined in "trials",\n'
                                    '           which selects withing the bounds and deltas defined.\n\n'
                                    'RANDOM_NO_BOUNDS:\n'
                                    '           Same as RANDOM, but does not use the defined deltas.\n'
                                    '           The deltas is set to 1 mm for all dimensions/thicknesses.\n\n'
                                    'ANYDETAIL:\n'
                                    '           Same as for ANYSMART, but will take some more time and\n'
                                    '           provide a chart of weight development during execution.\n\n'
                                    'PSO - Particle Swarm Search:\n'
                                    '           The information can be found on \n'
                                    '           http://pythonhosted.org/pyswarm/ \n'
                                    '           For further information google it!\n'
                                    '           Parameters:\n'
                                    '           swarmsize : The number of particles in the swarm (Default: 100)\n'
                                    '           omega : Particle velocity scaling factor (Default: 0.5)\n'
                                    '           phip : Scaling factor to search away from the particle’s \n'
                                    '                           best known position (Default: 0.5)\n'
                                    '           phig : Scaling factor to search away from the swarm’s best \n'
                                    '                           known position (Default: 0.5)\n'
                                    '           maxiter : The maximum number of iterations for the swarm \n'
                                    '                           to search (Default: 100)\n'
                                    '           minstep : The minimum stepsize of swarm’s best position \n'
                                    '                           before the search terminates (Default: 1e-8)\n'
                                    '           minfunc : The minimum change of swarm’s best objective value\n'
                                    '                           before the search terminates (Default: 1e-8)\n\n'
                            
                                    '\n'
                                    'All algorithms calculates local scantling and buckling requirements')

    def toggle(self):
        if self._toggle_btn.config('relief')[-1] == 'sunken':
            self._toggle_btn.config(relief="raised")
            self._toggle_btn.config(bg = 'salmon')

            predefined_stiffener_iter  = []
        else:
            self._toggle_btn.config(relief="sunken")
            self._toggle_btn.config(bg = 'salmon')
            self._toggle_btn.config(bg='lightgreen')

            predefined_stiffener_iter = []
            open_files = askopenfilenames(parent=self._frame, title='Choose files to open', initialdir=self._root_dir)
            if self._initial_cylinder_obj.LongStfObj is not None:
                predefined_stiffener_iter = hlp.helper_read_section_file(files=list(open_files),
                                                                         obj=self._initial_cylinder_obj.LongStfObj)

        if predefined_stiffener_iter == []:
            self._toggle_btn.config(relief="raised")
            self._toggle_btn.config(bg = 'salmon')

            self._predefined_stiffener_iter  = None
        else:
            self._predefined_stiffener_iter = predefined_stiffener_iter

        self.update_running_time()

    def open_example_file(self):
        import os
        if os.path.isfile('sections.csv'):
            os.startfile('sections.csv')
        else:
            os.startfile(self._root_dir + '/' + 'sections.csv')

    def show_calculated(self):
        ''' '''
        pass

    def plot_results(self):
        if len(self._opt_results) != 0:
            op.plot_optimization_results(self._opt_results)

    def write_result_csv(self):
        if len(self._opt_results) != 0:
            print(self._opt_results)


def receive_progress_info():
    '''
    Get progress info from optimization algorithm.
    :return:
    '''
    print('hi')



if __name__ == '__main__':
    root = tk.Tk()
    my_app = CreateOptimizeCylinderWindow(master=root)
    root.mainloop()







import pathlib
import tkinter as tk
from _tkinter import TclError
from tkinter.ttk import Combobox
import os

try:
    import any_files.example_data as test
    import any_files.helper as hlp
except ModuleNotFoundError:
    import ANYstructure.any_files.example_data as test
    import ANYstructure.any_files.helper as hlp


from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
# Implement the default Matplotlib key bindings.
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure

class CreateStructureWindow():
    '''
    This is the tkinter GUI for defining plate/stiffener properties.
    '''
    def __init__(self, master, app):
        super(CreateStructureWindow, self).__init__()
        self._frame = master
        self._frame.wm_title("Define structure properties")
        self._frame.geometry('1800x900')
        self._frame.grab_set()
        if __name__ == '__main__':
            self._initial_structure_obj = test.get_structure_calc_object()
            self._initial_calc_obj = test.get_structure_calc_object()

            self._section_list = []
            self._section_objects = []
            for section in hlp.helper_read_section_file('bulb_anglebar_tbar_flatbar.csv'):
                SecObj = Section(section)
                self._section_list = hlp.add_new_section(self._section_list, SecObj)
                self._section_objects.append(SecObj)
                # m = self._ent_section_list.children['menu']
                # m.add_command(label=SecObj.__str__(), command=self.section_choose)

            self._clicked_button = ["long stf", "ring stf", "ring frame", "flat long stf", 'flat stf', 'flat girder'][0]
        else:
            self.app = app
            self._clicked_button = app._clicked_section_create# if app._line_is_active else None
            try:
                if self._clicked_button in ['flat stf', "flat long stf"]:
                    self._initial_structure_obj =  self.app._line_to_struc[app._active_line][0].Stiffener
                elif self._clicked_button == 'flat girder':
                    self._initial_structure_obj = self.app._line_to_struc[app._active_line][5].Girder
                elif self._clicked_button in ["long stf"]:
                    self._initial_structure_obj =  self.app._line_to_struc[app._active_line][5].LongStfObj
                elif self._clicked_button == "ring stf":
                    self._initial_structure_obj = self.app._line_to_struc[app._active_line][5].RingStfObj
                elif self._clicked_button == "ring frame":
                    self._initial_structure_obj = self.app._line_to_struc[app._active_line][0].RingFrameObj
                else:
                    self._initial_structure_obj = None

            except (KeyError, AttributeError) as error:
                self._initial_structure_obj = None
            self._section_list = [section.__str__() for section in app._sections]
            self._section_objects = app._sections


        image_dir = os.path.dirname(__file__) + '\\images\\'
        self._opt_runned = False
        self._opt_resutls = ()
        self._draw_scale = 0.5
        self._canvas_dim = (500, 450)
        ent_w = 10
        start_x, start_y, dx, dy = 20, 70, 60, 33
        self._canvas_struc = tk.Canvas(self._frame, width=self._canvas_dim[0], height=self._canvas_dim[1],
                                       background='azure', relief='groove', borderwidth=2)
        self.structure_types = ['T','L', 'L-bulb','FB']
        self._canvas_struc.place(x=10, y=440)
        tk.Label(self._frame, text='-- Define structure properties here --', font='Verdana 15 bold').place(x=10, y=10)
        #
        # ### Adding matplotlib
        # fig, ax =  run_section_properties()# Figure(figsize=(4, 4), dpi=100)
        # t = np.arange(0, 3, .01)
        # #fig.add_subplot(111).plot(t, 2 * np.sin(2 * np.pi * t))
        #
        # canvas = FigureCanvasTkAgg(fig, master=master)  # A tk.DrawingArea.
        # canvas.draw()
        # canvas.get_tk_widget().place(x=start_x+17*dx, y=start_y+dy  )
        #
        # toolbar = NavigationToolbar2Tk(canvas, master)
        # toolbar.update()
        # canvas.get_tk_widget().place(x=start_x+17*dx, y=start_y+10*dy  )
        #
        # def on_key_press(event):
        #     print("you pressed {}".format(event.key))
        #     key_press_handler(event, canvas, toolbar)
        #
        # canvas.mpl_connect("key_press_event", on_key_press)
        
        self._new_spacing = tk.DoubleVar()
        self._new_pl_thk = tk.DoubleVar()
        self._new_web_h = tk.DoubleVar()
        self._new_web_thk = tk.DoubleVar()
        self._new_fl_w = tk.DoubleVar()
        self._new_fl_thk = tk.DoubleVar()
        self._new_stiffener_type = tk.StringVar()
        self._new_stiffener_filter = tk.StringVar()
        self._new_stiffener_filter.set('No filter applied')
        self._new_girder_length = tk.DoubleVar()
        self._new_section = tk.StringVar()


        self._ent_section_list = Combobox(self._frame, values = self._section_list, textvariable = self._new_section,
                                          width = 40)
        self._ent_section_list.bind("<<ComboboxSelected>>", self.section_choose)
        # self._ent_section_list = tk.OptionMenu(self._frame, self._new_section, command=self.section_choose,
        #                                        *['',] if self._section_list == [] else self._section_list)
        self._ent_structure_options = tk.OptionMenu(self._frame,self._new_stiffener_type,
                                                   command=self.option_choose,*self.structure_types)
        self._ent_filter_stf = tk.OptionMenu(self._frame,self._new_stiffener_filter,
                                                   command=self.regen_option_menu,*['No filter applied','L-bulb', 'L', 'FB', 'T'])

        self._ent_spacing = tk.Entry(self._frame, textvariable=self._new_spacing, width=ent_w)
        self._ent_pl_thk = tk.Entry(self._frame, textvariable=self._new_pl_thk, width=ent_w)
        self._ent_web_h = tk.Entry(self._frame, textvariable=self._new_web_h, width=ent_w)
        self._ent_web_thk = tk.Entry(self._frame, textvariable=self._new_web_thk, width=ent_w)
        self._ent_fl_w = tk.Entry(self._frame, textvariable=self._new_fl_w, width=ent_w)
        self._ent_fl_thk = tk.Entry(self._frame, textvariable=self._new_fl_thk, width=ent_w)
        self._ent_girder_length = tk.Entry(self._frame, textvariable=self._new_girder_length, width=ent_w)



        tk.Label(self._frame, text='Stiffener type:', font='Verdana 9 bold').place(x=start_x, y=start_y )
        tk.Label(self._frame, text='Girder length (Lg)', font='Verdana 9 bold').place(x=start_x+9*dx,
                                                                                     y=start_y + 15 * dy)
        tk.Label(self._frame, text='[m]', font='Verdana 9 bold').place(x=start_x + 14 * dx,y=start_y + 15 * dy)
        self._ent_girder_length.place(x=start_x + 12 * dx, y=start_y + 15 * dy)

        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x+3*dx, y=start_y+dy  )
        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x+3*dx, y=start_y + 2*dy)
        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x +3*dx, y=start_y + 3*dy)
        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x+3*dx, y=start_y + 4*dy)
        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x+3*dx, y=start_y + 5*dy)
        tk.Label(self._frame, text='[mm]', font='Verdana 9 bold').place(x=start_x+3*dx, y=start_y + 6*dy)

        tk.Label(self._frame, text='Existing sections:', font='Verdana 9 bold').place(x=start_x+4*dx, y=start_y + 6*dy)
        tk.Label(self._frame, text='filter ->', font='Verdana 9 bold').place(x=start_x + 4 * dx,
                                                                                      y=start_y + 7 * dy)

        self._ent_section_list.place(x=start_x+7*dx, y=start_y + 6*dy)
        self._ent_filter_stf.place(x=start_x+5*dx, y=start_y + 7*dy)

        tk.Button(self._frame, text='Read section list from file', command=self.read_sections, font='Verdana 10 bold',
                  bg = 'blue', fg = 'yellow').place(x=start_x+12*dx, y=start_y + 6*dy)
        tk.Button(self._frame, text='Load built in sections', command=self.read_sections_built_in, font='Verdana 10 bold',
                  bg = 'azure', fg = 'black').place(x=start_x+12*dx, y=start_y + 7*dy)
        # setting default values
        init_dim,init_thk = 0.05,0.002

        if self._initial_structure_obj != None:
            self._new_stiffener_type.set(self._initial_structure_obj.get_stiffener_type())
            self._new_spacing.set(self._initial_structure_obj.get_s()*1000)
            self._new_pl_thk.set(self._initial_structure_obj.get_pl_thk()*1000)
            self._new_web_h.set(self._initial_structure_obj.get_web_h()*1000)
            self._new_web_thk.set(self._initial_structure_obj.get_web_thk()*1000)
            self._new_fl_w.set(self._initial_structure_obj.get_fl_w()*1000)
            self._new_fl_thk.set(self._initial_structure_obj.get_fl_thk()*1000)
        else:
            self._new_spacing.set(0)
            self._new_pl_thk.set(0)
            self._new_web_h.set(0)
            self._new_web_thk.set(0)
            self._new_fl_w.set(0)
            self._new_fl_thk.set(0)

        self._new_girder_length.set(10)

        self._ent_structure_options.place(x=start_x + dx * 3, y=start_y)

        if self._new_spacing.get() != 0:
            tk.Label(self._frame, text='Spacing', font='Verdana 9').place(x=start_x, y=start_y + dy)
            self._ent_spacing.place(x=start_x + dx * 2, y=start_y+dy)
        if self._new_pl_thk.get() != 0:
            tk.Label(self._frame, text='Plate thk.', font='Verdana 9').place(x=start_x, y=start_y + 2 * dy)
            self._ent_pl_thk.place(x=start_x + dx * 2, y=start_y+2*dy)
        if self._new_web_h.get() != 0:
            tk.Label(self._frame, text='Web height', font='Verdana 9').place(x=start_x, y=start_y + 3 * dy)
            self._ent_web_h.place(x=start_x + dx * 2, y=start_y+3*dy)
        if self._new_web_thk.get() != 0:
            tk.Label(self._frame, text='Web thk.', font='Verdana 9').place(x=start_x, y=start_y + 4 * dy)
            self._ent_web_thk.place(x=start_x + dx * 2, y=start_y+4*dy)
        if self._new_fl_w.get() != 0:
            tk.Label(self._frame, text='Flange width', font='Verdana 9').place(x=start_x, y=start_y + 5 * dy)
            self._ent_fl_w.place(x=start_x + dx * 2, y=start_y+5*dy)
        if self._new_fl_thk.get() != 0:
            tk.Label(self._frame, text='Flange thk.', font='Verdana 9').place(x=start_x, y=start_y + 6 * dy)
            self._ent_fl_thk.place(x=start_x + dx * 2, y=start_y+6*dy)

        self._new_spacing.trace('w',self.draw_trace)
        self._new_pl_thk.trace('w',self.draw_trace)
        self._new_web_h.trace('w',self.draw_trace)
        self._new_web_thk.trace('w',self.draw_trace)
        self._new_fl_w.trace('w',self.draw_trace)
        self._new_fl_thk.trace('w',self.draw_trace)
        try:
            img_file_name = 'img_stiffened_plate_panel.gif'
            if os.path.isfile('images/' + img_file_name):
                file_path = 'images/' + img_file_name
            else:
                file_path = os.path.dirname(os.path.abspath(__file__)) + '/images/' + img_file_name
            photo = tk.PhotoImage(file=file_path)
            label = tk.Label(self._frame, image=photo)
            label.image = photo  # keep a reference!
            label.place(x=550, y=610)
        except TclError:
            pass
        try:
            img_file_name = 'img_T_L_FB.gif'
            if os.path.isfile('images/' + img_file_name):
                file_path = 'images/' + img_file_name
            else:
                file_path = os.path.dirname(os.path.abspath(__file__)) + '/images/' + img_file_name
            photo_T_L_FB = tk.PhotoImage(file=file_path)
            label = tk.Label(self._frame, image=photo_T_L_FB )
            label.image = photo_T_L_FB   # keep a reference!
            label.place(x=270, y=50)
        except TclError:
            pass

        # Close and save depending on input
        # "long stf", "ring stf", "ring frame", "flat long stf"
        if self._clicked_button is not None:
            self.close_and_save = tk.Button(self._frame, text='Click to return section data to ' + self._clicked_button,
                                            command=self.save_and_close, bg='green',
                                            font='Verdana 10 bold', fg='yellow')
            self.close_and_save.place(x=start_x + dx * 9, y=start_y + dy * 12)



        self.draw_properties()

    def regen_option_menu(self, event = None):
        self._ent_section_list.destroy()
        sections =  []
        if self._section_list == []:
            sections = ['',]
        elif self._new_stiffener_filter.get() == 'No filter applied':
            sections = self._section_list
        else:
            for sec_obj in self._section_objects:
                if sec_obj.stf_type == self._new_stiffener_filter.get():
                    sections.append(sec_obj.__str__())
        start_x, start_y, dx, dy = 20, 70, 60, 33
        # self._ent_section_list = tk.OptionMenu(self._frame, self._new_section, command=self.section_choose,
        #                                        *sections)
        self._ent_section_list = Combobox(self._frame, values=sections, textvariable=self._new_section, width = 40)
        self._ent_section_list.bind("<<ComboboxSelected>>", self.section_choose)
        self._ent_section_list.place(x=start_x + 7 * dx, y=start_y + 6 * dy)

        pass

    def option_choose(self, event):
        '''
        Action when the option menu is changed.
        :param event:
        :return:
        '''
        start_x, start_y, dx, dy = 20, 70, 50, 33

        tk.Label(self._frame, text='Spacing', font='Verdana 9').place(x=start_x, y=start_y + dy)
        self._ent_spacing.place(x=start_x + dx * 2, y=start_y+dy)

        tk.Label(self._frame, text='Plate thk.', font='Verdana 9').place(x=start_x, y=start_y + 2 * dy)
        self._ent_pl_thk.place(x=start_x + dx * 2, y=start_y+2*dy)

        tk.Label(self._frame, text='Web height', font='Verdana 9').place(x=start_x, y=start_y + 3 * dy)
        self._ent_web_h.place(x=start_x + dx * 2, y=start_y+3*dy)

        tk.Label(self._frame, text='Web thk.', font='Verdana 9').place(x=start_x, y=start_y + 4 * dy)
        self._ent_web_thk.place(x=start_x + dx * 2, y=start_y+4*dy)

        if self._new_stiffener_type.get()!='FB':
            tk.Label(self._frame, text='Flange width', font='Verdana 9').place(x=start_x, y=start_y + 5 * dy)
            self._ent_fl_w.place(x=start_x + dx * 2, y=start_y+5*dy)
        else: self._ent_fl_w.place_forget()
        if self._new_stiffener_type.get()!='FB':
            tk.Label(self._frame, text='Flange thk.', font='Verdana 9').place(x=start_x, y=start_y + 6 * dy)
            self._ent_fl_thk.place(x=start_x + dx * 2, y=start_y+6*dy)
        else: self._ent_fl_thk.place_forget()
        if self._new_stiffener_type.get()=='FB':
            self._new_fl_w.set(0)
            self._new_fl_thk.set(0)
        self.draw_properties()

    def checkered(self, line_distance):
        '''
        Grid lines in the properties canvas.
        :param line_distance: 
        :return: 
        '''
        # vertical lines at an interval of "line_distance" pixel
        for x in range(line_distance, self._canvas_dim[0], line_distance):
            self._canvas_struc.create_line(x, 0, x, self._canvas_dim[0], fill="grey", stipple='gray50')
        # horizontal lines at an interval of "line_distance" pixel
        for y in range(line_distance, self._canvas_dim[1], line_distance):
            self._canvas_struc.create_line(0, y, self._canvas_dim[0], y, fill="grey", stipple='gray50')

    def draw_properties(self):
        '''
        Drawing properties in the canvas.
        :return:
        '''
        self._canvas_struc.delete('all')
        self.checkered(10)
        ctr_x = self._canvas_dim[0] / 2
        ctr_y = self._canvas_dim[1] / 2 + 200
        m = self._draw_scale
        init_color, init_stipple = 'blue', 'gray50'

        try: spacing = self._new_spacing.get() 
        except TclError: spacing = 0
        try: pl_thk = self._new_pl_thk.get() 
        except TclError: pl_thk = 0        
        try: web_h = self._new_web_h.get()
        except TclError: web_h = 0
        try: web_thk = self._new_web_thk.get()
        except TclError: web_thk = 0
        try: fl_w = self._new_fl_w.get() 
        except TclError: fl_w = 0
        try: fl_thk = self._new_fl_thk.get()
        except TclError: fl_thk = 0

        self._canvas_struc.create_rectangle(0, 0, self._canvas_dim[0] + 10, 70, fill='white')

        self._canvas_struc.create_text(250, 15, text='Plate: ' + str(spacing ) + 'x' +
                                                    str(pl_thk ),font='Verdana 10 bold',fill='black')
        self._canvas_struc.create_rectangle(ctr_x - m * spacing / 2, ctr_y,ctr_x + m * spacing / 2,
                                           ctr_y - m * pl_thk, fill='black', stipple=init_stipple)

        self._canvas_struc.create_text(250, 35, text='Web: ' + str(web_h ) + 'x'+ str(web_thk )
                                  ,font='Verdana 10 bold',fill='blue')
        self._canvas_struc.create_rectangle(ctr_x - m * web_thk / 2,ctr_y - m * pl_thk,ctr_x + m * web_thk / 2,
                                           ctr_y - m * (web_h+ pl_thk), fill='blue', stipple=init_stipple)

        self._canvas_struc.create_text(250, 55, text='Flange: '+ str(fl_w ) + 'x'+ str(fl_thk ),
                                  font='Verdana 10 bold',fill='red')
        if self._new_stiffener_type.get() in ['L', 'L-bulb']:
            self._canvas_struc.create_rectangle(ctr_x - m * web_thk / 2, ctr_y- m * (pl_thk + web_h),ctr_x + m * fl_w,
                                               ctr_y - m * (pl_thk + web_h + fl_thk),fill='red', stipple=init_stipple)
        else:
            self._canvas_struc.create_rectangle(ctr_x - m * fl_w / 2, ctr_y- m * (pl_thk + web_h),ctr_x + m * fl_w / 2,
                                               ctr_y - m * (pl_thk + web_h + fl_thk),fill='red', stipple=init_stipple)

    def draw_trace(self,*args):
        '''
        Updating when values in entered
        :param event:
        :return:
        '''
        self.draw_properties()

    def save_and_close(self):
        '''
        Save and close
        :return:
        '''
        if __name__ == '__main__':
            self._frame.destroy()
            return

        self.app.on_close_structure_window([float(num) for num in [self._new_spacing.get(),self._new_pl_thk.get(),
                                                                   self._new_web_h.get(),self._new_web_thk.get(),
                                                                   self._new_fl_w.get(),self._new_fl_thk.get()]] +
                                           [self._new_stiffener_type.get(), self._clicked_button])
        self._frame.destroy()

    def section_choose(self, event = None):
        ''' Choosing a section. '''
        #chosen_section = self._new_section.get()
        chosen_section = event.widget.get()
        for section in self._section_objects:
            if chosen_section == section.__str__():
                self._new_web_h.set(section.stf_web_height*1000)
                self._new_web_thk.set(section.stf_web_thk*1000)
                self._new_fl_w.set(section.stf_flange_width*1000)
                self._new_fl_thk.set(section.stf_flange_thk*1000)
                self._new_stiffener_type.set(section.stf_type)
        self.option_choose(None)

    def read_sections(self):
        '''
        Read a list.
        '''
        from tkinter import filedialog
        import helper as hlp
        from pathlib import Path

        file = filedialog.askopenfile('r')
        file = Path(file.name)
        #m = self._ent_section_list.children['menu']

        for section in hlp.helper_read_section_file(file.name):
            SecObj = Section(section)
            self._section_list = hlp.add_new_section(self._section_list, SecObj)
            self._section_objects.append(SecObj)
            #m.add_command(label=SecObj.__str__(), command=self.section_choose)

    def read_sections_built_in(self):
        '''
        Read a list.
        '''
        import helper as hlp
        if pathlib.Path('bulb_anglebar_tbar_flatbar.csv').exists():
            libfile = 'bulb_anglebar_tbar_flatbar.csv'
        else:
            libfile = 'bulb_anglebar_tbar_flatbar.csv'
        for section in hlp.helper_read_section_file(libfile):
            SecObj = Section(section)
            self._section_list = hlp.add_new_section(self._section_list, SecObj)
            self._section_objects.append(SecObj)
            #m.add_command(label=SecObj.__str__(), command=self.section_choose)

        self.regen_option_menu()

class Section:
    '''
    Creates a section property.
    'stf_type': [self._new_stf_type.get(), ''],
    'stf_web_height': [self._new_stf_web_h.get()/1000, 'm'],
    'stf_web_thk': [self._new_sft_web_t.get()/1000, 'm'],
    'stf_flange_width': [self._new_stf_fl_w.get()/1000, 'm'],
    'stf_flange_thk': [self._new_stf_fl_t.get()/1000, 'm'],
    '''
    def __init__(self, input_dict):
        super(Section, self).__init__()
        self._stf_type = input_dict['stf_type'] if type(input_dict['stf_type']) != list \
            else input_dict['stf_type'][0]
        self._stf_web_height = input_dict['stf_web_height']if type(input_dict['stf_web_height']) != list \
            else input_dict['stf_web_height'][0]
        self._stf_web_thk = input_dict['stf_web_thk']if type(input_dict['stf_web_thk']) != list \
            else input_dict['stf_web_thk'][0]
        self._stf_flange_width = input_dict['stf_flange_width']if type(input_dict['stf_flange_width']) != list \
            else input_dict['stf_flange_width'][0]
        self._stf_flange_thk = input_dict['stf_flange_thk']if type(input_dict['stf_flange_thk']) != list \
            else input_dict['stf_flange_thk'][0]

    def __str__(self):
        ''' Returning a string. '''
        base_name = self.stf_type+ '_' + str(round(self.stf_web_height*1000, 0)) + 'x' + \
                   str(round(self.stf_web_thk*1000, 0))
        if self._stf_type == 'FB':
            ret_str = base_name
        elif self._stf_type in ['L-bulb', 'bulb', 'hp']:
            ret_str = 'Bulb'+str(int(self.stf_web_height*1000 + self.stf_flange_thk*1000))+'x'+\
                      str(round(self.stf_web_thk*1000, 0))+ '__' +str(round(self.stf_web_height*1000, 0)) + 'x' + \
                   str(round(self.stf_web_thk*1000, 0))+ str(round(self.stf_flange_width*1000, 0)) + 'x' + \
                      str(round(self.stf_flange_thk*1000, 0))
        else:
            ret_str = base_name + '__' + str(round(self.stf_flange_width*1000, 0)) + 'x' + \
                      str(round(self.stf_flange_thk*1000, 0))

        ret_str = ret_str.replace('.', '_')

        return ret_str


    @property
    def stf_type(self):
        return self._stf_type

    @stf_type.setter
    def stf_type(self, value):
        self._stf_type = value

    @property
    def stf_web_height(self):
        return self._stf_web_height

    @stf_web_height.setter
    def stf_web_height(self, value):
        self._stf_web_height = value

    @property
    def stf_web_thk(self):
        return self._stf_web_thk

    @stf_web_thk.setter
    def stf_web_thk(self, value):
        self._stf_web_thk = value

    @property
    def stf_flange_width(self):
        return self._stf_flange_width

    @stf_flange_width.setter
    def stf_flange_width(self, value):
        self._stf_flange_width = value

    @property
    def stf_flange_thk(self):
        return self._stf_flange_thk

    @stf_flange_thk.setter
    def stf_flange_thk(self, value):
        self._stf_flange_thk = value

    def return_puls_input(self):
        '''
        Returns as input good for PULS
        :return:
        '''
        return {'Stiffener type (L,T,F)': self.stf_type,  'Stiffener boundary': 'C',
                'Stiff. Height': self.stf_web_height*1000,
                   'Web thick.': self.stf_web_thk*1000, 'Flange width': self.stf_flange_width*1000,
                          'Flange thick.': self.stf_flange_thk*1000}

# def run_section_properties(pl_s = 0.75, pl_t = 0.015, hw = 0.4, tw = 0.018, bf = 0.15, tf = 0.02):
#     import sectionproperties.pre.sections as sections
#     from sectionproperties.analysis.cross_section import CrossSection
#     from matplotlib import pyplot as plt
#
#     # create a 50 diameter circle discretised by 64 points
#     geometry = sections.MonoISection(
#         d=(pl_t+hw+tf)*1000, b_t=bf*1000, b_b=pl_s*1000, t_ft=tf*1000, t_fb=pl_t*1000, t_w=tw*1000, r=8, n_r=16
#     )
#     mesh = geometry.create_mesh(mesh_sizes=[3.0])
#     section = CrossSection(geometry, mesh)  # create a CrossSection object
#     mesh_nodes = section.mesh_nodes
#     mesh_elements = section.mesh_elements
#     # plot the mesh
#     (fig, ax) = plt.subplots(figsize=(4, 4), dpi=100)
#     ax.triplot(mesh_nodes[:, 0], mesh_nodes[:, 1], mesh_elements[:, 0:3], lw=0.5)
#     # #section.display_mesh_info()  # display the mesh information
#     # ax = section.plot_mesh(pause=True)  # plot the generated mesh
#     #
#     # # perform a geometric, warping and plastic analysis, displaying the time info
#     # section.calculate_geometric_properties(time_info=True)
#     # section.calculate_warping_properties(time_info=True)
#     # section.calculate_plastic_properties(time_info=True)
#     #
#     # # print the results to the terminal
#     # section.display_results()
#     #
#     # # get the second moments of area and the torsion constant
#     # (ixx_c, iyy_c, ixy_c) = section.get_ic()
#     # j = section.get_j()
#     #
#     # # print the sum of the second moments of area and the torsion constant
#     # print("Ixx + Iyy = {0:.3f}".format(ixx_c + iyy_c))
#     # print("J = {0:.3f}".format(j))
#     return fig, ax
#
#



if __name__ == '__main__':

    # sec1 = Section({'stf_type': 'T', 'stf_web_height': 0.35, 'stf_web_thk': 0.02, 'stf_flange_width': 0.15,
    #                 'stf_flange_thk': 0.015})
    #
    # sec_list = [sec1, Section({'stf_type': 'FB', 'stf_web_height': 0.35, 'stf_web_thk': 0.02, 'stf_flange_width': 0,
    #                 'stf_flange_thk': 0}), Section({'stf_type': 'T', 'stf_web_height': 0.4, 'stf_web_thk': 0.02,
    #                                                     'stf_flange_width': 0.15, 'stf_flange_thk': 0.02})]
    #
    # hlp.add_new_section(sec_list, sec1)
    # run_section_properties()
    root = tk.Tk()
    my_app = CreateStructureWindow(root, app=None)
    root.mainloop()



'''
Main file for handeling interface toward DNVGL software.
Consist mainly of helper functions.
'''


try:
    import any_files.pl_stf_window as plstf
    import any_files.helper as hlp
except ModuleNotFoundError:
    import ANYstructure.any_files.pl_stf_window as plstf
    import ANYstructure.any_files.helper as hlp
def point_to_js_command(point_coord, point_name):
    '''
    Returning a js script.
    :param point_coord:
    :param point_name:
    :return:
    '''
    return point_name + ' = Point('+str(point_coord[0])+', 0, '+str(point_coord[1]) + ');\n'

def line_to_js_command_reference(from_point, to_point, curve_name):
    '''
    Returning a js script based on reference modelling.
    :param from_point:
    :param to_point:
    :param curve_name:
    :return:
    '''
    return curve_name +' = CreateLineTwoPoints(' + 'point'+str(from_point)+','+ ' point' + \
           str(to_point) + ');\n'

def section_property_to_js(section: plstf.Section = None):
    '''
    Sct3 = BarSection(0.25, 0.015);
    Sct1 = UnsymISection(0.35, 0.02, 0, 0, 0, 0.15, 0.075, 0.02);
    Sct2 = LSection(0.3, 0.012, 0.1, 0.02);
    '''

    if section.stf_type == 'T':
        js_def =  'UnsymISection('+str(section.stf_web_height)+', '+str(section.stf_web_thk)+', 0, 0, 0, '+ \
                  str(section.stf_flange_width)+', '+str(section.stf_flange_width/2)+', '+\
                  str(section.stf_flange_thk)+');\n'

    elif section.stf_type in ['L', 'L-bulb']:
        js_def =  'LSection('+str(section.stf_web_height)+', '+str(section.stf_web_thk)+', '+ \
                  str(section.stf_flange_width)+', '+ str(section.stf_flange_thk)+');\n'
    else:
        js_def =  'BarSection('+str(section.stf_web_height)+', '+str(section.stf_web_thk)+');\n'
    ret_str = section.__str__() +  ' = ' + js_def
    ret_str.replace('-', '_')
    return ret_str

class JSfile:
    '''
    An object representation of the js file.
    '''
    def __init__(self, points, lines, sections: plstf.Section = None, line_to_struc = None):
        super(JSfile, self).__init__()
        self._output_lines = list()
        self._points = points
        self._lines = lines
        self._sections = sections
        self._line_to_struc = line_to_struc

    @property
    def output_lines(self):
        return self._output_lines

    def write_points(self):
        ''' Writing point in the point list. '''
        for point_name, point_coord in self._points.items():
            self._output_lines.append(point_to_js_command(point_coord, point_name))

    def write_lines(self):
        ''' Writing the line list. '''
        for line_name, point_to_point in self._lines.items():
            self._output_lines.append(line_to_js_command_reference(point_to_point[0], point_to_point[1], line_name))

    def write_sections(self):
        ''' Exporting all sections. '''
        for section in self._sections:
            self._output_lines.append(section_property_to_js(section))

    def write_beams(self):
        '''
        Making beams and assining properties.
        Bm1 = Beam(line3);
        Bm1.section = T_400_0x12_0__250_0x14_0;
        '''
        for line_name, line_prop in self._line_to_struc.items():
            if line_prop[0].Stiffener is not None:
                beam_name = 'ANYbm'+str(hlp.get_num(line_name))
                self.output_lines.append(beam_name+' = Beam('+line_name+');\n')
                section = plstf.Section(line_prop[0].Stiffener.get_structure_prop())
                self.output_lines.append(beam_name + '.section = '+section.__str__()+';\n')

if __name__ == '__main__':
    import example_data as test
    from tkinter import filedialog

    imp_file = open('test_js.js', 'w')

    JS = JSfile(test.get_point_dict(), test.get_line_dict(), test.get_section_list(),
                line_to_struc=test.get_line_to_struc())
    JS.write_points()
    JS.write_lines()
    JS.write_sections()
    JS.write_beams()

    imp_file.writelines(JS.output_lines)
    imp_file.close()






# This is fatigue properties are defined.
import tkinter as tk

try:
    import any_files.example_data as test
    import any_files.SN_curve_parameters as sn
except ModuleNotFoundError:
    import ANYstructure.any_files.example_data as test
    import ANYstructure.any_files.SN_curve_parameters as sn


class CreateFatigueWindow():
    '''
    This class initiates the GUI used to define stresses for the selected structure.
    '''
    def __init__(self, master, app=None):
        super(CreateFatigueWindow, self).__init__()
        if __name__ == '__main__':
            self._initial_structure_obj = test.get_structure_object()
            self.load_objects = [test.get_loa_fls_load(),test.get_loa_uls_load(),
                                 test.get_bal_fls_load(), test.get_bal_uls_load()]
            self.active_line = 'line1'
            points = test.line_dict['line1']
            coords = (test.point_dict['point'+str(points[0])], test.point_dict['point'+str(points[1])])
            self.pressure_coords = self.get_pressure_point_coord_from_two_points(coords[0],coords[1])
            self.comp_objects = [test.get_tank_object()]
            self._initial_fatigue_obj = test.get_fatigue_object()

        else:
            if app._line_to_struc[app._active_line][0] is None:
                return
            elif app._line_to_struc[app._active_line][0].Stiffener is None:
                return
            self.app = app
            self.active_line = app._active_line
            points = app._line_dict[self.active_line]
            coords = (app._point_dict['point'+str(points[0])], app._point_dict['point'+str(points[1])])
            self.pressure_coords = self.get_pressure_point_coord_from_two_points(coords[0], coords[1])
            self._initial_structure_obj = app._line_to_struc[app._active_line][0].Stiffener
            self.load_objects = app._line_to_struc[app._active_line][3]

            self.comp_objects = [app._tank_dict['comp'+str(comp_i)] for comp_i in
                                 app.get_compartments_for_line(app._active_line)]
            self._initial_fatigue_obj = app._line_to_struc[self.active_line][2]


        self._frame = master
        self._frame.wm_title("Specify fatigue properties here.")
        self._frame.geometry('1300x810')
        self._frame.grab_set()

        tk.Label(self._frame, text='-- Fatigue calculation for plates according to DNVGL-RP-C203, '
                                   'Section 5 Simplified fatigue analysis --',
                 font='Verdana 15 bold').place(x=10, y=10)

        ent_w = 10

        self.new_sn_curve = tk.StringVar()
        self.new_k_factor = tk.DoubleVar()
        self.new_no_of_cycles = tk.DoubleVar()
        self.new_design_life = tk.DoubleVar()
        self.new_dff = tk.DoubleVar()

        self.new_weibull_loa = tk.DoubleVar()
        self.new_weibull_bal = tk.DoubleVar()
        self.new_weibull_prt = tk.DoubleVar()

        self.new_period_loa = tk.DoubleVar()
        self.new_period_bal = tk.DoubleVar()
        self.new_period_prt = tk.DoubleVar()

        self.new_corr_loc_loa = tk.DoubleVar()
        self.new_corr_loc_bal = tk.DoubleVar()
        self.new_corr_loc_prt = tk.DoubleVar()

        self.new_fraction_loa = tk.DoubleVar()
        self.new_fraction_bal = tk.DoubleVar()
        self.new_fraction_prt = tk.DoubleVar()

        self.new_az_loa = tk.DoubleVar()
        self.new_az_bal = tk.DoubleVar()
        self.new_az_prt = tk.DoubleVar()


        sn_curves = sn.get_all_curves()

        self.ent_sn_curve = tk.OptionMenu(self._frame, self.new_sn_curve, command=self.change_sn_curve,*sn_curves)
        self.ent_dff = tk.Entry(self._frame, textvariable=self.new_dff, width=ent_w)
        self.ent_k_factor = tk.Entry(self._frame, textvariable=self.new_k_factor, width=ent_w)
        self.ent_no_of_cycles = tk.Entry(self._frame, textvariable=self.new_no_of_cycles, width=ent_w)
        self.ent_new_design_life = tk.Entry(self._frame, textvariable=self.new_design_life, width=ent_w)

        self.ent_weibull_loa = tk.Entry(self._frame, textvariable=self.new_weibull_loa, width=ent_w)
        self.ent_weibull_bal = tk.Entry(self._frame, textvariable=self.new_weibull_bal, width=ent_w)
        self.ent_weibull_prt = tk.Entry(self._frame, textvariable=self.new_weibull_prt, width=ent_w)

        self.ent_period_loa = tk.Entry(self._frame, textvariable=self.new_period_loa, width=ent_w)
        self.ent_period_bal = tk.Entry(self._frame, textvariable=self.new_period_bal, width=ent_w)
        self.ent_period_prt = tk.Entry(self._frame, textvariable=self.new_period_prt, width=ent_w)

        self.ent_corr_loc_loa = tk.Entry(self._frame, textvariable=self.new_corr_loc_loa, width=ent_w)
        self.ent_corr_loc_bal = tk.Entry(self._frame, textvariable=self.new_corr_loc_bal, width=ent_w)
        self.ent_corr_loc_prt = tk.Entry(self._frame, textvariable=self.new_corr_loc_prt, width=ent_w)

        self.ent_fraction_loa = tk.Entry(self._frame, textvariable=self.new_fraction_loa, width=ent_w)
        self.ent_fraction_bal = tk.Entry(self._frame, textvariable=self.new_fraction_bal, width=ent_w)
        self.ent_fraction_prt = tk.Entry(self._frame, textvariable=self.new_fraction_prt, width=ent_w)

        self.ent_acc_loa = tk.Entry(self._frame, textvariable=self.new_az_loa, width=ent_w)
        self.ent_acc_bal = tk.Entry(self._frame, textvariable=self.new_az_bal, width=ent_w)
        self.ent_acc_prt = tk.Entry(self._frame, textvariable=self.new_az_prt, width=ent_w)

        start_x, start_y, dx, dy = 20, 100, 150, 35

        loaded_exist = False
        ballast_exist = False
        part_exist = False
        fls_exist = (loaded_exist, ballast_exist, part_exist)

        # Frames to hide loaded, ballast or part
        loa_fr = tk.Frame(self._frame, width=100, height=170, bg="gray25", colormap="new")
        bal_fr = tk.Frame(self._frame, width=100, height=170, bg="gray25", colormap="new")
        prt_fr = tk.Frame(self._frame, width=100, height=170, bg="gray25", colormap="new")

        for load in self.load_objects:
            if load != None:
                if load.get_limit_state() == 'FLS':
                    if load.get_load_condition() == 'loaded':
                        loaded_exist = True
                    elif load.get_load_condition() == 'ballast':
                        ballast_exist = True
                    elif load.get_load_condition() == 'part':
                        part_exist = True
                    else:
                        pass
            fls_exist = (loaded_exist, ballast_exist, part_exist)


        if self._initial_fatigue_obj == None:
            self.new_sn_curve.set('Ec')
            self.new_k_factor.set(1)
            self.new_no_of_cycles.set(10000)
            self.new_design_life.set(20)
            self.new_dff.set(2)
            if any(fls_exist):
                if loaded_exist:
                    self.new_fraction_loa.set(1 / fls_exist.count(True))
                    self.new_weibull_loa.set(0.8)
                    self.new_period_loa.set(8)
                    self.new_corr_loc_loa.set(0.5)
                    self.new_az_loa.set(0.5)
                if ballast_exist:
                    self.new_fraction_bal.set(1 / fls_exist.count(True))
                    self.new_weibull_bal.set(0.8)
                    self.new_period_bal.set(8)
                    self.new_corr_loc_bal.set(0.5)
                    self.new_az_bal.set(0.5)
                if part_exist:
                    self.new_fraction_prt.set(1 / fls_exist.count(True))
                    self.new_weibull_prt.set(0.8)
                    self.new_period_prt.set(8)
                    self.new_corr_loc_prt.set(0.5)
                    self.new_az_prt.set(0.5)
        else:
            fat_prop = self._initial_fatigue_obj.get_fatigue_properties()
            self.new_sn_curve.set(fat_prop['SN-curve'])
            self.new_k_factor.set(fat_prop['SCF'])
            self.new_no_of_cycles.set(fat_prop['n0'])
            self.new_design_life.set(fat_prop['Design life'])
            self.new_dff.set(fat_prop['DFF'])
            if any(fls_exist):
                if loaded_exist:
                    self.new_fraction_loa.set(fat_prop['Fraction'][0])
                    self.new_weibull_loa.set(fat_prop['Weibull'][0])
                    self.new_period_loa.set(fat_prop['Period'][0])
                    self.new_corr_loc_loa.set(fat_prop['CorrLoc'][0])
                    self.new_az_loa.set(fat_prop['Accelerations'][0])
                if ballast_exist:
                    self.new_fraction_bal.set(fat_prop['Fraction'][1])
                    self.new_weibull_bal.set(fat_prop['Weibull'][1])
                    self.new_period_bal.set(fat_prop['Period'][1])
                    self.new_corr_loc_bal.set(fat_prop['CorrLoc'][1])
                    self.new_az_bal.set(fat_prop['Accelerations'][1])
                if part_exist:
                    self.new_fraction_prt.set(fat_prop['Fraction'][2])
                    self.new_weibull_prt.set(fat_prop['Weibull'][2])
                    self.new_period_prt.set(fat_prop['Period'][2])
                    self.new_corr_loc_prt.set(fat_prop['CorrLoc'][2])
                    self.new_az_prt.set(fat_prop['Accelerations'][2])

        all_acc = (self.new_az_loa.get(), self.new_az_bal.get(), self.new_az_prt.get())

        count = 1
        for load in self.load_objects:
            if load == None:
                continue
            press = []
            if load.get_limit_state() == 'FLS':
                tk.Label(self._frame, text=str(count)+'. '+load.get_name(), font='Verdana 8')\
                    .place(x=start_x + 5 * dx, y=start_y + (5+count) * dy)
                idx = 0
                for exist in fls_exist:
                    if exist:
                        press.append(round(load.get_calculated_pressure(self.pressure_coords,all_acc[idx],
                                                                        self._initial_structure_obj.
                                                                        get_structure_type()), 1))
                        idx += 1

                tk.Label(self._frame, text=press, font='Verdana 8') \
                    .place(x=start_x + 6.5 * dx, y=start_y + (5 + count) * dy)
                count += 1
        press = []
        for comp in self.comp_objects:
            if comp == None:
                continue
            tk.Label(self._frame, text=str(count) + '. ' +str(comp.get_name()), font='Verdana 8') \
                .place(x=start_x + 5 * dx, y=start_y + (5 + count) * dy)

            idx = 0

            if fls_exist[0] and comp.is_loaded_condition():
                press.append(round(comp.get_calculated_pressure(self.pressure_coords, all_acc[0]), 1))
            if fls_exist[1] and comp.is_ballast_condition():
                press.append(round(comp.get_calculated_pressure(self.pressure_coords, all_acc[1]), 1))
            if fls_exist[2] and any([comp.is_loaded_condition(),comp.is_ballast_condition()]):
                press.append(round(comp.get_calculated_pressure(self.pressure_coords, all_acc[2]), 1))

            tk.Label(self._frame, text=press, font='Verdana 8') \
                .place(x=start_x + 6.5 * dx, y=start_y + (5 + count) * dy)

            count += 1

        tk.Label(self._frame, text='Design life:', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 0*dy)
        tk.Label(self._frame, text='Design Fatigue Factor (DFF):', font='Verdana 8 bold') \
            .place(x=start_x+ 3*dx , y=start_y + 0*dy)
        tk.Label(self._frame, text='SN-curve:', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 1*dy)
        tk.Label(self._frame, text='Cycles in return period, n0', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 2*dy)
        tk.Label(self._frame, text='Stress Concentration Factor, SCF', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 3*dy)

        tk.Label(self._frame, text='Loaded', font='Verdana 8 bold') \
            .place(x=start_x+2*dx , y=start_y + 5*dy)
        tk.Label(self._frame, text='Ballast', font='Verdana 8 bold') \
            .place(x=start_x+3*dx , y=start_y + 5*dy)
        tk.Label(self._frame, text='Part', font='Verdana 8 bold') \
            .place(x=start_x+4*dx , y=start_y + 5*dy)
        tk.Label(self._frame, text='Defined loads', font='Verdana 8 bold') \
            .place(x=start_x+5*dx , y=start_y + 5*dy)
        tk.Label(self._frame, text='Resulting pressures', font='Verdana 8 bold') \
            .place(x=start_x+6.5*dx , y=start_y + 5*dy)

        tk.Label(self._frame, text='Fraction (sum of bal/part/loa is 1)', font='Verdana 8 bold') \
            .place(x=start_x, y=start_y + 6 * dy)
        tk.Label(self._frame, text='Weibull', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 7*dy)
        tk.Label(self._frame, text='Period', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 8*dy)
        tk.Label(self._frame, text='Corr. loc.', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 9*dy)
        tk.Label(self._frame, text='Accelerations', font='Verdana 8 bold') \
            .place(x=start_x , y=start_y + 10*dy)

        self.ent_new_design_life.place(x=start_x+2*dx, y=start_y + 0 * dy)
        self.ent_dff.place(x=start_x + 5 * dx, y=start_y + 0 * dy)
        self.ent_sn_curve.place(x=start_x+2*dx , y=start_y + 1*dy)
        self.ent_no_of_cycles .place(x=start_x+2*dx , y=start_y + 2*dy)
        self.ent_k_factor.place(x=start_x+2*dx , y=start_y + 3*dy)

        self.ent_fraction_loa.place(x=start_x + 2 * dx, y=start_y + 6 * dy)
        self.ent_fraction_bal.place(x=start_x+3*dx , y=start_y + 6*dy)
        self.ent_fraction_prt.place(x=start_x+4*dx , y=start_y + 6*dy)

        self.ent_weibull_loa.place(x=start_x + 2 * dx, y=start_y + 7 * dy)
        self.ent_weibull_bal.place(x=start_x+3*dx , y=start_y + 7*dy)
        self.ent_weibull_prt.place(x=start_x+4*dx , y=start_y + 7*dy)

        self.ent_period_loa.place(x=start_x + 2 * dx, y=start_y + 8 * dy)
        self.ent_period_bal.place(x=start_x+3*dx, y=start_y + 8*dy)
        self.ent_period_prt.place(x=start_x+4*dx, y=start_y + 8*dy)

        self.ent_corr_loc_loa.place(x=start_x + 2 * dx, y=start_y + 9 * dy)
        self.ent_corr_loc_bal.place(x=start_x+3*dx, y=start_y + 9*dy)
        self.ent_corr_loc_prt.place(x=start_x+4*dx, y=start_y + 9*dy)

        self.ent_acc_loa.place(x=start_x + 2 * dx, y=start_y + 10 * dy)
        self.ent_acc_bal.place(x=start_x + 3 * dx, y=start_y + 10 * dy)
        self.ent_acc_prt.place(x=start_x + 4 * dx, y=start_y + 10 * dy)

        # if not loaded_exist:
        #     loa_fr.place(x=start_x + 2 * dx, y=start_y + 6 * dy)
        # elif not ballast_exist:
        #     bal_fr.place(x=start_x + 3 * dx, y=start_y + 6 * dy)
        # elif not part_exist:
        #     prt_fr.place(x=start_x + 4 * dx, y=start_y + 6 * dy)

        self._close_and_save = tk.Button(self._frame, text='Return fatigue parameters',
                                        command=self.save_and_close, bg='green', font='Verdana 15', fg='yellow')
        self._close_and_save.place(x=start_x + dx, y=start_y + dy * 15)

    def get_pressure_point_coord_from_two_points(self,p1,p2):
        ''' Finding the coordinates to use in pressure calculations '''

        if p1[1] <= p2[1]:
            start_point = p1
            end_point = p2
        else:
            start_point = p2
            end_point = p1

        vector = [end_point[0]-start_point[0], end_point[1]-start_point[1]]

        return start_point[0]+vector[0]*1/3, start_point[1]+vector[1]*1/3

    def change_sn_curve(self,event):
        '''
        Action when changing the structure type
        :return:
        '''
        self.new_sn_curve.set(event)

    def save_and_close(self):
        '''
        Save and close
        :return:
        '''
        if __name__ == '__main__':
            self._frame.destroy()
            return
        # order of return is (Loaded,Ballast,Part)
        temp_dict = {'SN-curve':self.new_sn_curve.get(),
                     'SCF':self.new_k_factor.get(),
                     'Design life':self.new_design_life.get(),
                     'n0':self.new_no_of_cycles.get(),
                     'Weibull':(self.new_weibull_loa.get(),self.new_weibull_bal.get(),self.new_weibull_prt.get()),
                     'Period':(self.new_period_loa.get(),self.new_period_bal.get(),self.new_period_prt.get()),
                     'Fraction':(self.new_fraction_loa.get(),self.new_fraction_bal.get(),self.new_fraction_prt.get()),
                     'CorrLoc':(self.new_corr_loc_loa.get(),self.new_corr_loc_bal.get(),self.new_corr_loc_prt.get()),
                     'Order':('Loaded','Ballast','Part'),
                     'Accelerations':(self.new_az_loa.get(),self.new_az_bal.get(),self.new_az_prt.get()),
                     'DFF': self.new_dff.get()}

        self.app.on_close_fatigue_window(temp_dict)
        self._frame.destroy()


if __name__ == '__main__':
    root = tk.Tk()
    my_app = CreateFatigueWindow(root,app=None)
    root.mainloop()


'''
Contains examples to be used for testing.
'''


ex1 = {'Identification': 'Panel 1', 'Length of panel': 3000, 'Stiffener spacing': 700, 'Plate thickness': 15,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 350, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 18, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}

ex2 = {'Identification': 'Panel 2', 'Length of panel': 3500, 'Stiffener spacing': 700, 'Plate thickness': 15,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 350, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 18, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}

ex3 = {'Identification': 'Panel 3', 'Length of panel': 4000, 'Stiffener spacing': 700, 'Plate thickness': 15,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 350, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 18, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}

ex4 = {'Identification': 'Panel 4', 'Length of panel': 3000, 'Stiffener spacing': 700, 'Plate thickness': 15,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 400, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 18, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}

ex5 = {'Identification': 'Panel 5', 'Length of panel': 3500, 'Stiffener spacing': 700, 'Plate thickness': 15,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 350, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 25, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}

ex6 = {'Identification': 'Panel 6', 'Length of panel': 4000, 'Stiffener spacing': 800, 'Plate thickness': 20,
       'Number of primary stiffeners': 20, 'Stiffener type (L,T,F)': 'T', 'Stiffener boundary': 'C',
       'Stiff. Height': 350, 'Web thick.': 12,'Flange width': 150, 'Flange thick.': 18, 'Tilt angle': 0,
       'Number of sec. stiffeners': 0, 'Modulus of elasticity': 2.1e11/1e6, "Poisson's ratio": 0.33,
       'Yield stress plate': 355, 'Yield stress stiffener': 355, 'Axial stress': 80, 'Trans. stress 1': 60,
       'Trans. stress 2': 50, 'Shear stress': 10, 'Pressure (fixed)': 0.08, 'In-plane support': 'Int'}


import pprint
from calc_structure import *
try:
    from any_files.calc_structure import *
except ModuleNotFoundError:
    from ANYstructure.any_files.calc_structure import *

structure_types = {'vertical': ['BBS', 'SIDE_SHELL', 'SSS'],
                         'horizontal': ['BOTTOM', 'BBT', 'HOPPER', 'MD'],
                         'non-wt': ['FRAME', 'GENERAL_INTERNAL_NONWT'],
                         'internals': ['INNER_SIDE', 'FRAME_WT', 'GENERAL_INTERNAL_WT',
                                       'INTERNAL_ZERO_STRESS_WT', 'INTERNAL_LOW_STRESS_WT']}

obj_dict = {'mat_yield': [355e6, 'Pa'], 
            'mat_factor': [1.15, ''],
            'span': [3.7, 'm'],
            'spacing': [0.75, 'm'],
            'plate_thk': [0.018, 'm'],
            'stf_web_height': [0.4, 'm'],
            'stf_web_thk': [0.012, 'm'],
            'stf_flange_width': [0.25, 'm'],
            'stf_flange_thk': [0.014, 'm'],
            'structure_type': ['BOTTOM', ''],
            'plate_kpp': [1, ''],
            'stf_kps': [1, ''],
            'stf_km1': [12, ''],
            'stf_km2': [24, ''],
            'stf_km3': [12, ''],
            'sigma_y1': [100, 'MPa'],
            'sigma_y2': [100, 'MPa'],
            'sigma_x2': [102.7, 'MPa'],
            'sigma_x1': [102.7, 'MPa'],
            'tau_xy': [5, 'MPa'],
            'stf_type': ['T', ''],
            'structure_types': [structure_types, ''],
            'zstar_optimization': [True, ''],
            'puls buckling method':[1,''],
            'puls boundary':['Int',''],
            'puls stiffener end':['C',''],
            'puls sp or up':['SP',''],
            'puls up boundary' :['SSSS',''],
            'panel or shell': ['panel', ''],
            'pressure side': ['both sides', ''],
            'girder_lg': [5, 'm']}

obj_dict_unitless = {'mat_yield': [355e6, ''], 'mat_factor': [1.15, ''],'span': [3.7, ''], 'spacing': [0.75, ''],
            'plate_thk': [0.018, ''],
            'stf_web_height': [0.4, ''], 'stf_web_thk': [0.012, ''], 'stf_flange_width': [0.25, ''],
            'stf_flange_thk': [0.014, ''], 'structure_type': ['BOTTOM', ''], 'plate_kpp': [1, ''],
            'stf_kps': [1, ''], 'stf_km1': [12, ''], 'stf_km2': [24, ''], 'stf_km3': [12, ''],
            'sigma_y1': [100, ''], 'sigma_y2': [100, ''], 'sigma_x2': [102.7, ''], 'sigma_x1': [102.7, ''],
            'tau_xy': [5, ''],
            'stf_type': ['T', ''], 'structure_types': [structure_types, ''], 'zstar_optimization': [True, ''],
            'puls buckling method':[1,''], 'puls boundary':['Int',''], 'puls stiffener end':['C',''],
            'puls sp or up':['SP',''], 'puls up boundary' :['SSSS',''], 'panel or shell': ['panel', ''],
            'pressure side': ['both sides', ''], 'girder_lg': [5, '']}

obj_dict_heavy = {'mat_yield': [355e6, 'Pa'], 'mat_factor': [1.15, ''],'span': [3700, 'm'], 'spacing': [0.75, 'm'],
            'plate_thk': [0.018, 'm'],
            'stf_web_height': [0.500, 'm'], 'stf_web_thk': [0.0120, 'm'], 'stf_flange_width': [0.150, 'm'],
            'stf_flange_thk': [0.02, 'm'], 'structure_type': ['BOTTOM', ''], 'plate_kpp': [1, ''],
            'stf_kps': [1, ''], 'stf_km1': [12, ''], 'stf_km2': [24, ''], 'stf_km3': [12, ''],
            'sigma_y1': [80, 'MPa'], 'sigma_y2': [80, 'MPa'], 'sigma_x2': [80, 'MPa'], 'sigma_x1': [80, 'MPa'], 'tau_xy': [5, 'MPa'],
            'stf_type': ['T', ''], 'structure_types': [structure_types, ''], 'zstar_optimization': [True, ''],
                  'puls buckling method':[2,''], 'puls boundary':['Int',''], 'puls stiffener end':['C',''],
            'puls sp or up':['SP',''], 'puls up boundary' :['SSSS',''], 'panel or shell': ['panel', ''],
            'pressure side': ['both sides', '']}

prescriptive_main_dict = dict()
prescriptive_main_dict['minimum pressure in adjacent spans']  = [None, '']
prescriptive_main_dict['material yield']  = [355e6, 'Pa']
prescriptive_main_dict['load factor on stresses']  = [1, '']
prescriptive_main_dict['load factor on pressure']  = [1, '']
prescriptive_main_dict['buckling method']  = ['ultimate', '']
prescriptive_main_dict['stiffener end support']  = ['Continuous', '']  # 'Continuous'
prescriptive_main_dict['girder end support']  = ['Continuous', '']  # 'Continuous'
prescriptive_main_dict['tension field']  = ['not allowed', '']  # 'not allowed'
prescriptive_main_dict['plate effective agains sigy']   = [True, ''] # True
prescriptive_main_dict['buckling length factor stf']  = [None, '']
prescriptive_main_dict['buckling length factor girder']  = [None, '']
prescriptive_main_dict['km3']  = [12, '']  # 12
prescriptive_main_dict['km2']  = [24, '']  # 24
prescriptive_main_dict['stiffener distance between lateral support']  = [None, '']
prescriptive_main_dict['girder distance between lateral support']  = [None, '']
# the following doesn't seem to exist in the class anymore
#prescriptive_main_dict['kgirder']  = [None, '']
prescriptive_main_dict['panel length, Lp']  = [None, '']
prescriptive_main_dict['pressure side']  = ['both sides', '']# either 'stiffener', 'plate', 'both'
prescriptive_main_dict['fabrication method stiffener'] =  ['welded', '']
prescriptive_main_dict['fabrication method girder'] =   ['welded', '']
prescriptive_main_dict['calculation domain'] = ['Flat plate, stiffened', '']

def main():
    # Prescriptive buckling UPDATED
    Plate = CalcScantlings(obj_dict_unitless)
    Stiffener = CalcScantlings(obj_dict_unitless)
    Girder = CalcScantlings(obj_dict_heavy)
    PreBuc = AllStructure(Plate = Plate, Stiffener = Stiffener, Girder = Girder,
                                  main_dict=prescriptive_main_dict)
    print('Results from scantling calculation for plate:')
    print(Plate)
    print('Results from scantling calculation for stiffener:')
    print(Stiffener)
    print('Results from scantling calculation for girder:')
    print(Girder)

    PreBuc.lat_press = 0.412197

    print('Results from buckling calculation for stiffened plate:')
    bucklingResult = PreBuc.plate_buckling()
    #pprint.pprint(bucklingResult)
    for key, value in bucklingResult.items():
        print(key, ': ')
        for key2, value2 in value.items():
            print('\t',key2, ': ', value2)

if __name__ == '__main__':
    main()


import any_files.example_data as test
from any_files.calc_loads import *
from any_files.helper import *

# try:
#     import any_files.example_data as test
#     from any_files.calc_loads import *
#     from any_files.helper import *
# except ModuleNotFoundError:
#     import ANYstructure.any_files.example_data as test
#     from ANYstructure.any_files.calc_loads import *
#     from ANYstructure.any_files.helper import *

import tkinter as tk
from tkinter import messagebox



class CreateLoadWindow():
    '''
    This class defines the external pressures on the hull (static and dynamic).
    '''
    @staticmethod
    def return_dummy_manual(line):
        load = 'manual'
        combination = 'manual'
        load_comb_dict = {}
        load_comb_dict[(combination, line, load)] = [tk.DoubleVar(), tk.DoubleVar(), tk.IntVar()]
        load_comb_dict[(combination, line, load)][0].set(0)
        load_comb_dict[(combination, line, load)][1].set(1)
        load_comb_dict[(combination, line, load)][2].set(1)
        return load_comb_dict

    def __init__(self, master,app=None):

        super(CreateLoadWindow, self).__init__()

        limit_states = ['ULS', 'FLS']
        if __name__ == '__main__':
            options_cond = ['loaded', 'ballast', 'tanktest', 'part', 'slamming']
            self._load_factors_dict = {'dnva': [1.3, 1.2, 0.7], 'dnvb': [1, 1, 1.3], 'tanktest': [1, 1, 0]}
            self._load_objects = {}
            self._load_comb_dict = {}
            self._line_dict = test.get_line_dict()
            self._load_count = 0
            self._slamming_load_count = 0
            self._point_dict = test.get_point_dict()
            self._canvas_scale = 20
            self._structure_types = {'vertical': ['BBS', 'SIDE_SHELL', 'SSS'],
                                     'horizontal': ['BOTTOM', 'BBT', 'HOPPER', 'MD']}

        else:
            self.app = app
            self._load_factors_dict = app._load_factors_dict
            options_cond = app._load_conditions
            self._load_objects = app._load_dict
            self._load_comb_dict = app._new_load_comb_dict
            self._line_dict = app._line_dict
            self._load_count = 0
            self._point_dict = app._point_dict
            self._canvas_scale = app._canvas_scale
            self._structure_types = app._structure_types
            self._slamming_load_count = 0

        self._point_is_active = False
        self._active_point = ''
        self._point_is_active = False
        self._active_lines = []
        self._add_to_lines = True
        self._lines_add_to_load = []

        listbox_select = 'extended'
        frame_dim = (1500,980)
        self._canvas_origo = (50,720-50)
        self._canvas_dim = (1000,720)

        self._frame = master
        self._frame.wm_title("Load properties")
        self._frame.geometry(str(frame_dim[0])+'x'+str(frame_dim[1]))
        self._frame.grab_set()

        self._frame.protocol("WM_DELETE_WINDOW", self.on_closing)

        tk.Frame(self._frame, width=5, height=980, bg="black", colormap="new").place(x =450, y = 0)
        tk.Frame(self._frame, width=455, height=5, bg="black", colormap="new").place(x = 0, y = 320)
        tk.Frame(self._frame, width=5, height=190, bg="black", colormap="new").place(x =1000, y = 0)
        tk.Frame(self._frame, width=5, height=190, bg="black", colormap="new").place(x=1250, y=0)
        tk.Frame(self._frame, width=1100, height=5, bg="black", colormap="new").place(x = 450, y = 190)

        # Main canvas creation
        self._main_canvas = tk.Canvas(self._frame,width=self._canvas_dim[0], height=self._canvas_dim[1],
                                     background='azure', relief = 'groove', borderwidth=2)
        self._main_canvas.place(relx=0.32,rely=0.25)
        self._global_shrink = 1
        base_canvas_dim = [1000, 720]  # do not modify this, sets the "orignal" canvas dimensions.
        self._canvas_dim = [int(base_canvas_dim[0] *self._global_shrink),
                           int(base_canvas_dim[1] *self._global_shrink)]
        self._canvas_base_origo = [50, base_canvas_dim[1] - 50] # 50 bottom left location of the canvas, (0,0)
        self._canvas_draw_origo = list(self._canvas_base_origo)
        self._previous_drag_mouse = list(self._canvas_draw_origo)

        # --- slider (used to zoom) ----
        self._slider = tk.Scale(self._frame,from_=60,to = 1, command = self.slider_used,
                               background='azure', relief = 'groove', borderwidth=2)
        self._slider.set(self._canvas_scale)
        self._slider.place(relx=0.32,rely=0.25)

        # --- Dynamic load input ---
        ent_x = 200
        delta_y = 30
        options_cond = tuple(options_cond)
        load_vert_start = 90
        tk.Label(self._frame, text='1. Dynamic loads', font='Verdana 10 bold', fg = 'red')\
            .place(x=10, y=load_vert_start - 80)
        tk.Label(self._frame, text='Define dynamic loads as an polynominal curve.\n'
                                  'Can be third degree, second degree, linear or constant \n'
,
                 font="Verdana 8 bold",justify = tk.LEFT).place(x=10, y=load_vert_start - 50)

        tk.Button(self._frame, text='Create dynamic load',  command=self.create_dynamic_load_object,
                  font='Verdana 9 bold', fg='yellow', bg = 'green' )\
            .place(x=270, y=load_vert_start + delta_y *6)

        self.close_window= tk.Button(self._frame, text='Press this to: \n'
                                                      'Save loads and \n'
                                                      'close the load window. ',font="Verdana 9 bold",
                                     command=self.save_and_close, bg = 'green', fg = 'yellow')
        self.close_window.place(x=ent_x*6.35, y=load_vert_start-20)

        self._new_dynamic_load_name = tk.StringVar()
        self._new_dynamic_load_name.set('load' + str(self._load_count))
        self._new_load_poly_third = tk.DoubleVar()
        self._new_load_poly_second = tk.DoubleVar()
        self._new_load_poly_first = tk.DoubleVar()
        self._new_load_poly_const = tk.DoubleVar()
        self._new_load_manual_pressure = tk.DoubleVar()
        self._new_dyn_load_condition = tk.StringVar()
        self._new_limit_state = tk.StringVar()
        self._new_limit_state.set('ULS')
        self._new_slamming_pressure = tk.DoubleVar()
        self._new_slamming_pressure_name = tk.StringVar()
        self._new_slamming_pressure_name.set('slamming')
        self._new_slamming_pl_mult = tk.DoubleVar()
        self._new_slamming_stf_mult = tk.DoubleVar()
        self._new_slamming_pl_mult.set(1.0)
        self._new_slamming_stf_mult.set(1.0)

        ent_w = 15
        ent_dyn_load_name = tk.Entry(self._frame, textvariable=self._new_dynamic_load_name, width=ent_w*2)
        ent_load_poly_third = tk.Entry(self._frame, textvariable=self._new_load_poly_third, width=ent_w)
        ent_load_poly_second = tk.Entry(self._frame, textvariable=self._new_load_poly_second, width=ent_w)
        ent_load_poly_first = tk.Entry(self._frame, textvariable=self._new_load_poly_first,width=ent_w)
        ent_load_poly_constant = tk.Entry(self._frame, textvariable=self._new_load_poly_const,width=ent_w)
        ent_load_condition = tk.OptionMenu(self._frame, self._new_dyn_load_condition, *options_cond)
        ent_limit_state = tk.OptionMenu(self._frame, self._new_limit_state, *limit_states)

        # Slamming pressures
        slx, sly = ent_x*5.6, load_vert_start-40
        tk.Label(self._frame,text = 'Load name:').place(x = slx-90, y = sly)
        ent_slamming_pressure = tk.Entry(self._frame, textvariable=self._new_slamming_pressure, width=ent_w)
        ent_slamming_pressure.place(x = slx, y = sly+delta_y)

        ent_slamming_pl_mult = tk.Entry(self._frame, textvariable=self._new_slamming_pl_mult, width=7)
        ent_slamming_pl_mult.place(x = slx + 50, y = sly + 1.8*delta_y)

        ent_slamming_stf_mult = tk.Entry(self._frame, textvariable=self._new_slamming_stf_mult, width=7)
        ent_slamming_stf_mult.place(x = slx + 50, y = sly+2.6*delta_y)

        tk.Label(self._frame,text='Pressure [Pa]:').place(x=slx - 90, y=sly+delta_y)
        tk.Label(self._frame, text='Plate multiplier, Ppl').place(x=slx - 90, y=sly + 1.8*delta_y)
        tk.Label(self._frame, text='Stiffener multiplier, Pst:').place(x=slx - 90, y=sly + 2.6*delta_y)
        ent_slamming_pressure_name = tk.Entry(self._frame, textvariable=self._new_slamming_pressure_name, width=ent_w)
        ent_slamming_pressure_name.place(x=slx, y=sly)
        tk.Button(self._frame, text = 'Create slamming load', command = self.create_slamming_load,
                  font='Verdana 9 bold', fg='yellow', bg = 'green' ) \
            .place(x=slx - 80, y=sly + 3.5*delta_y)

        ent_dyn_load_name.place(x=ent_x, y=load_vert_start + 0 * delta_y)
        ent_load_poly_third.place(x=ent_x, y=load_vert_start + 1 * delta_y)
        ent_load_poly_second.place(x=ent_x, y=load_vert_start + 2 * delta_y)
        ent_load_poly_first.place(x=ent_x, y=load_vert_start +3 * delta_y)
        ent_load_poly_constant.place(x=ent_x, y=load_vert_start + 4 * delta_y)
        ent_load_condition.place(x=ent_x - 5, y=load_vert_start + 5 * delta_y - 5)
        ent_limit_state.place(x=ent_x - 5, y=load_vert_start + 6 * delta_y - 5)

        tk.Label(self._frame, text='Input load name:').place(x=10, y=load_vert_start + 0*delta_y)
        tk.Label(self._frame, text='Third degree poly [x^3]').place(x=10, y=load_vert_start+delta_y)
        tk.Label(self._frame, text='Second degree poly [x^2]').place(x=10, y=load_vert_start + 2*delta_y)
        tk.Label(self._frame, text='First degree poly [x]').place(x=10, y=load_vert_start + 3*delta_y)
        tk.Label(self._frame, text='Constant [C]').place(x=10, y=load_vert_start + 4 * delta_y)
        tk.Label(self._frame, text='Load condition').place(x=10, y=load_vert_start + 5 * delta_y)
        tk.Label(self._frame, text='Limit state').place(x=10, y=load_vert_start + 6 * delta_y)

        # --- Static load input ---
        horizontal_start = 500
        tk.Label(self._frame, text='2. Static loads', font='Verdana 10 bold', fg = 'red') \
            .place(x=horizontal_start, y=load_vert_start - 80)
        tk.Label(self._frame, text='3. Slamming pressure', font='Verdana 10 bold', fg = 'red') \
            .place(x=horizontal_start+520, y=load_vert_start - 80)
        tk.Label(self._frame, text = 'Hydrostatic loads defined by draft.',
                 font="Verdana 8 bold")\
            .place(x = horizontal_start,y = load_vert_start-1.5*delta_y)
        tk.Label(self._frame, text = 'Define static draft from sea [m]:')\
            .place(x = horizontal_start,y = load_vert_start + delta_y)
        tk.Label(self._frame, text='Define name of static load:').place(x=horizontal_start,y=load_vert_start)
        tk.Label(self._frame, text='Select load condition:').place(x=horizontal_start,y=load_vert_start + delta_y*2)

        self._new_static_load_name = tk.StringVar()
        self._new_static_draft = tk.DoubleVar()
        self._new_static_condition = tk.StringVar()
        self._new_static_load_name.set('static'+str(self._load_count))

        tk.Entry(self._frame, textvariable = self._new_static_load_name,width=ent_w)\
            .place(x = horizontal_start+200, y = load_vert_start,)
        tk.Entry(self._frame, textvariable = self._new_static_draft,width=ent_w)\
            .place(x = horizontal_start+200, y = load_vert_start + delta_y)
        tk.OptionMenu(self._frame, self._new_static_condition, *options_cond)\
            .place(x = horizontal_start+200, y = load_vert_start + 2*delta_y)
        tk.Button(self._frame, text = 'Create static load', command = self.create_static_load_object,
                  font='Verdana 9 bold', fg='yellow', bg = 'green' )\
            .place(x = horizontal_start + 340, y = load_vert_start )

        # --- showing created loads ---
        start_y = 340
        tk.Label(self._frame, text='3. Created loads are seen below\n'
                                   '(scroll if not all is shown.)\n'
                                   'DOUBLE CLICK load to see assosiated lines.:',
                 font="Verdana 10 bold", fg='red').place(x=10, y=start_y)
        self._load_obj_box = tk.Listbox(self._frame, height = 15, selectmode = listbox_select, bg='azure',
                                       relief = 'groove', borderwidth=2)
        self._load_obj_box.place(x=10, y=start_y + 3 * delta_y)
        self._load_obj_box.bind('<<ListboxSelect>>', self.left_click_load_box)
        loads_scrollbar = tk.Scrollbar(self._frame)
        loads_scrollbar.config(command = self._load_obj_box.yview)
        loads_scrollbar.place(x=140, y=start_y + 3 * delta_y)
        self._load_obj_box.config(yscrollcommand=loads_scrollbar.set)
        tk.Label(self._frame, text = 'Select to see assosiated lines: ').place(x=10, y=start_y +  2*delta_y)

        # --- showing the lines applied to the load above ---
        self._load_obj_lines_box = tk.Listbox(self._frame, height = 15, selectmode = listbox_select, bg = 'azure',
                                             relief = 'groove', borderwidth=2)
        lines_scrollbar = tk.Scrollbar(self._frame)
        lines_scrollbar.config(command = self._load_obj_lines_box.yview)
        lines_scrollbar.place(x=330, y=start_y+ 3 * delta_y)
        self._load_obj_lines_box.config(yscrollcommand=lines_scrollbar.set)

        self._load_obj_lines_box.place(x=200, y=start_y+ 3 * delta_y)
        self._load_obj_lines_box.bind('<<ListboxSelect>>', self.left_click_load_box)
        tk.Label(self._frame, text = '-->',font="Verdana 8 bold").place(x=160, y= load_vert_start + 15 * delta_y )

        # --- dropdown meny to choose load to assosiate with lines ---
        self._load_options = ['']
        self._new_assisiate_load = tk.StringVar()
        self._ent_assosiate_load = tk.OptionMenu(self._frame, self._new_assisiate_load, *tuple(self._load_options))
        self._ent_assosiate_load.place(relx=0.85,rely=0.21)

        # --- Button to assosiate selecte lines to load
        tk.Button(self._frame, text = 'Press to add selected lines to selecte load',
                  command=self.append_line_to_load, fg = 'yellow', bg='green',font='Verdana 9 bold')\
            .place(relx=0.32,rely=0.215)
        tk.Label(self._frame,text='Select a load in "3." to and then choose lines to apply to load\n '
                                 '(select by clicking lines). Alterntively define manually ------>')\
            .place(relx=0.56,rely=0.205)

        # --- delete a created load ---
        tk.Button(self._frame, text="Delete selected load",command=self.delete_load,
                  font='Verdana 9 bold', fg='yellow', bg = 'red' )\
            .place(x=10, y=start_y + 12 * delta_y)

        # --- updating the imported loads from main window ---
        if len(self._load_objects) > 0:
            self.import_update()

        # --- properties canvas to show variables for load ---
        self._canvas_properties = tk.Canvas(self._frame, height=200, width=350,
                                           background='azure', relief = 'groove', borderwidth=2)
        self._canvas_properties.place(x= 10, y = load_vert_start + delta_y*22.5)

        self.controls()
        self.draw_canvas()

    def delete_load(self):
        self._load_objects.pop(self._load_obj_box.get('active'))
        self._load_obj_box.delete('active')

    def slider_used(self, event):
        '''
        Action when slider is activated.
        :return:
        '''
        self._canvas_scale = self._slider.get()
        self.draw_canvas()

    def draw_canvas(self, load_selected=False):
        '''
        Making the line canvas
        :return:
        '''
        self._main_canvas.delete('all')

        # grid for the canavs

        self._main_canvas.create_line(self._canvas_draw_origo[0], 0, self._canvas_draw_origo[0], self._canvas_dim[1],
                                     stipple='gray50')
        self._main_canvas.create_line(0, self._canvas_draw_origo[1], self._canvas_dim[0], self._canvas_draw_origo[1],
                                     stipple='gray50')
        self._main_canvas.create_text(self._canvas_draw_origo[0] - 30 ,
                                     self._canvas_draw_origo[1] + 20 , text='(0,0)',
                                     font='Text 10')
        self._main_canvas.create_text([800 ,50],
                                     text='Mouse left click:  select lines to loads\n'
                                          'Mouse right click: clear all selection\n'
                                          'Shift key press: add selected line\n'
                                          'Control key press: remove selected line', font='Verdana 8 bold',
                                     fill='red')
        # drawing the line dictionary.
        if len(self._line_dict) != 0:
            for line, value in self._line_dict.items():
                color = 'black'
                coord1 = self.get_point_canvas_coord('point' + str(value[0]))
                coord2 = self.get_point_canvas_coord('point' + str(value[1]))

                vector = [coord2[0] - coord1[0], coord2[1] - coord1[1]]
                # drawing a bold line if it is selected
                if line in self._active_lines:
                    self._main_canvas.create_line(coord1, coord2, width=6, fill='orange')
                    self._main_canvas.create_text(coord1[0] + vector[0] / 2 + 5, coord1[1] + vector[1] / 2 + 10,
                                                 text='Line ' + str(get_num(line)), font='Verdand 10 bold',
                                                 fill='red')
                else:
                    self._main_canvas.create_line(coord1, coord2, width=3, fill=color)
                    self._main_canvas.create_text(coord1[0] - 20 + vector[0] / 2 + 5, coord1[1] + vector[1] / 2 + 10,
                                                 text='line' + str(get_num(line)), font="Text 8", fill='black')

    def button_2_click_and_drag(self,event):

        self._canvas_draw_origo = (self._canvas_draw_origo[0]-(self._previous_drag_mouse[0]-event.x),
                                  self._canvas_draw_origo[1]-(self._previous_drag_mouse[1]-event.y))

        self._previous_drag_mouse = (event.x,event.y)
        self.draw_canvas()

    def mouse_scroll(self,event):
        self._canvas_scale +=  event.delta/50
        self._canvas_scale = 0 if self._canvas_scale < 0 else self._canvas_scale

        self.draw_canvas()

    def get_loads(self):
        '''
        Returning loads
        :return:
        '''
        return self._load_objects

    def make_load_comb_dict(self, line, load):
        '''
        Making the load comb dict
        (comb,line,load) : [DoubleVar(),DoubleVar(), IntVar()] #
        # {'dnva':[1.3,1.2,0.7], 'dnvb':[1,1,1.3], 'tanktest':[1,1,1]} # DNV  loads factors
        :return:
        '''

        for combination in self._load_factors_dict.keys():
            factors =self._load_factors_dict[combination]
            if load != 'manual':# and (combination, line, load) not in self._load_comb_dict.keys():
                #print(combination, line, load)
                self._load_comb_dict[(combination, line, load)] = [tk.DoubleVar(), tk.DoubleVar(), tk.IntVar()]
                if combination != 'tanktest':
                    if self._load_objects[load][0].is_static():
                        if self._load_objects[load][0].get_limit_state() == 'FLS':
                            self._load_comb_dict[(combination, line, load)][0].set(0)
                        elif self._load_objects[load][0].is_tank_test():
                            self._load_comb_dict[(combination, line, load)][0].set(0)
                            self._load_comb_dict[(combination, line, load)][2].set(0)
                        else:
                            self._load_comb_dict[(combination, line, load)][0].set(factors[0])
                            self._load_comb_dict[(combination, line, load)][2].set(1)
                        self._load_comb_dict[(combination, line, load)][1].set(0)

                    else:
                        self._load_comb_dict[(combination, line, load)][0].set(0)
                        if self._load_objects[load][0].get_limit_state() == 'FLS':
                            self._load_comb_dict[(combination, line, load)][1].set(1)
                        else:
                            self._load_comb_dict[(combination, line, load)][1].set(factors[2])
                        self._load_comb_dict[(combination, line, load)][2].set(1)
                else:
                    if self._load_objects[load][0].is_tank_test():
                        self._load_comb_dict[(combination, line, load)][0].set(1)
                        self._load_comb_dict[(combination, line, load)][1].set(0)
                        self._load_comb_dict[(combination, line, load)][2].set(1)
                    else:
                        self._load_comb_dict[(combination, line, load)][0].set(0)
                        self._load_comb_dict[(combination, line, load)][1].set(0)
                        self._load_comb_dict[(combination, line, load)][2].set(0)
            else:
                combination = 'manual'
                self._load_comb_dict[(combination, line, load)] = [tk.DoubleVar(), tk.DoubleVar(), tk.IntVar()]
                self._load_comb_dict[(combination, line, load)][0].set(0)
                self._load_comb_dict[(combination, line, load)][1].set(1)
                self._load_comb_dict[(combination, line, load)][2].set(1)

    def create_dynamic_load_object(self, slamming_load = False):
        '''
        Creating load object for the selected lines.
        '(poly_third = None,poly_second = None, poly_first = None, poly_const = None
        , load_condition = None, structure_type = None, man_press = None, static_draft = None)'
        :return:
        '''

        variables = ['poly_third','poly_second', 'poly_first', 'poly_const', 'load_condition',
                     'man_press', 'static_draft', 'name_of_load', 'limit_state', 'structure_types',
                     'slamming mult pl', 'slamming mult stf']
        existing_load = None
        if not slamming_load:
            name_of_load = self._new_dynamic_load_name.get()
            if name_of_load in self._load_objects.keys():
                # Existing load
                existing_load = copy.deepcopy(self._load_objects[name_of_load])
                self._load_objects.pop(name_of_load)
                self._load_obj_box.delete(0,'end')
                for load in self._load_objects.keys():
                    self._load_obj_box.insert('end', load)

            values = [self._new_load_poly_third.get(),self._new_load_poly_second.get(),
                      self._new_load_poly_first.get(),self._new_load_poly_const.get(),
                      self._new_dyn_load_condition.get(), None, None, name_of_load,
                      self._new_limit_state.get(), self._structure_types, 1, 1]
        else:
            name_of_load = self._new_slamming_pressure_name.get()
            if name_of_load in self._load_objects.keys():
                existing_load = copy.deepcopy(self._load_objects[name_of_load])
                self._load_objects.pop(name_of_load)
                self._load_obj_box.delete(0, 'end')
                for load in self._load_objects.keys():
                    self._load_obj_box.insert('end', load)

            values = [0, 0, 0, self._new_slamming_pressure.get(),
                      'slamming', None, None, name_of_load,
                      None, self._structure_types,
                      self._new_slamming_pl_mult.get(),
                      self._new_slamming_stf_mult.get()]

        count_i = 0
        current_load_dict = {}
        for item in variables:
            current_load_dict[item] = values[count_i]
            count_i += 1

        self._load_objects[name_of_load] = [Loads(current_load_dict),[] if existing_load is None else existing_load[1]]
        self._load_options.append(name_of_load)
        self._ent_assosiate_load.destroy()
        self._ent_assosiate_load = tk.OptionMenu(self._frame, self._new_assisiate_load, *tuple(self._load_options))
        self._ent_assosiate_load.place(relx=0.85,rely=0.21)
        self._load_obj_box.insert('end',name_of_load)
        if not slamming_load:
            self._load_count += 1
            self._new_dynamic_load_name.set('load'+str(self._load_count))
        else:
            self._new_slamming_pressure_name.set('slamming' + str(self._slamming_load_count))
            self._slamming_load_count += 1

    def create_slamming_load(self):
        ''' Creates a slamming load object. '''
        self.create_dynamic_load_object(slamming_load=True)

    def create_static_load_object(self):
        '''
        Creating static loads.
        '(poly_third = None,poly_second = None, poly_first = None, poly_const = None
        , load_condition = None, structure_type = None, man_press = None, static_draft = None)'
        :return:
        '''
        variables = ['poly_third','poly_second', 'poly_first', 'poly_const', 'load_condition',
                     'structure_type', 'man_press', 'static_draft','name_of_load']

        name_of_load = self._new_static_load_name.get()
        existing_load = None

        if name_of_load in self._load_objects.keys():
            existing_load = copy.deepcopy(self._load_objects[name_of_load])
            self._load_objects.pop(name_of_load)
            self._load_obj_box.delete(0,'end')
            for load in self._load_objects.keys():
                self._load_obj_box.insert('end', load)

        values = [None,None,None,None,self._new_static_condition.get(),None,None,
                  self._new_static_draft.get(), name_of_load]

        count_i = 0
        current_load_dict = {}

        for item in variables:
            current_load_dict[item] = values[count_i]
            count_i += 1

        self._load_objects[name_of_load] = [Loads(current_load_dict),[] if existing_load is None else existing_load[1]]

        self._load_options.append(name_of_load)
        self._ent_assosiate_load.destroy()
        self._ent_assosiate_load = tk.OptionMenu(self._frame, self._new_assisiate_load, *tuple(self._load_options))
        self._ent_assosiate_load.place(relx=0.85,rely=0.21)

        self._load_obj_box.insert('end', name_of_load)
        self._load_count += 1
        self._new_static_load_name.set('static' + str(self._load_count))

    def append_line_to_load(self):
        '''
        Specifying lines for the load
        :return:
        '''

        current_load = self._new_assisiate_load.get()
        if current_load != '':
            self._load_objects[current_load][1] = []
            for line in self._active_lines:
                #if line not in self._load_objects[current_load][1]:
                self._load_objects[current_load][1].append(line)
        else:
            mess = tk.messagebox.showwarning('Select load',message='Select a load to apply to the selected lines.',
                                             type='ok')

    def save_and_close(self):
        '''
        Save and close
        :return:
        '''
        if __name__ == '__main__':
            self._frame.destroy()
            return

        for load, data in self._load_objects.items():
            for line in data[1]:
                self.make_load_comb_dict(line,load)

        for line in self.app._line_dict.keys():
            self.make_load_comb_dict(line,'manual')
        if self._load_objects is not None:

            self.app.on_close_load_window(self._load_objects, self._load_count, self._load_comb_dict)
        self._frame.destroy()

    def on_closing(self):
        '''
        Action when closing the window without saving.
        :return:
        '''
        if __name__ == '__main__':
            self._frame.destroy()
            return

        mess = tk.messagebox.showwarning('Closed without saving', 'Closing will not save loads you have created',
                                  type = 'okcancel')
        if mess == 'ok':
            self._frame.grab_release()
            self._frame.destroy()
            self.app.on_aborted_load_window()

    def get_point_canvas_coord(self, point_no):
        '''
        Returning the canvas coordinates of the point. This value will change with slider.
        '''

        point_coord_x = self._canvas_draw_origo[0] + self._point_dict[point_no][0] * self._canvas_scale
        point_coord_y = self._canvas_draw_origo[1] - self._point_dict[point_no][1] * self._canvas_scale

        return [point_coord_x, point_coord_y]

    def controls(self):
        '''
        Specifying the controls to be used.
        :return:
        '''
        self._main_canvas.bind('<Button-1>', self.left_click)
        self._main_canvas.bind('<Button-2>', self.button_2_click)
        self._main_canvas.bind('<Button-3>', self.right_click)
        self._load_obj_box.bind('<Button-1>', self.left_click_load_box)
        self._frame.bind('<Shift_L>', self.shift_pressed)
        self._frame.bind('<Shift_R>', self.shift_pressed)
        self._frame.bind('<Control_L>', self.ctrl_pressed)
        self._frame.bind('<Control_R>', self.ctrl_pressed)
        self._main_canvas.bind("<MouseWheel>", self.mouse_scroll)
        self._main_canvas.bind("<B2-Motion>", self.button_2_click_and_drag)

    def shift_pressed(self,event=None):
        '''
        Event is executed when shift key pressed.
        :return:
        '''
        self._add_to_lines = True

    def ctrl_pressed(self,event=None):
        '''
        Event when control is pressed.
        :param event:
        :return:
        '''
        self._add_to_lines = False

    def button_2_click(self, event):
        '''
        Event when right click.
        :param evnet:
        :return:
        '''
        self._previous_drag_mouse = [event.x, event.y]

    def left_click(self, event):
        '''
        When clicking the right button, this method is called.
        method is referenced in
        '''
        self._previous_drag_mouse = [event.x, event.y]
        click_x = self._main_canvas.winfo_pointerx() - self._main_canvas.winfo_rootx()
        click_y = self._main_canvas.winfo_pointery() - self._main_canvas.winfo_rooty()
        stop = False

        if len(self._line_dict) > 0:
            for key, value in self._line_dict.items():

                coord1x = self.get_point_canvas_coord('point' + str(value[0]))[0]
                coord2x = self.get_point_canvas_coord('point' + str(value[1]))[0]
                coord1y = self.get_point_canvas_coord('point' + str(value[0]))[1]
                coord2y = self.get_point_canvas_coord('point' + str(value[1]))[1]

                vector = [coord2x - coord1x, coord2y - coord1y]
                click_x_range = [ix for ix in range(click_x - 10, click_x + 10)]
                click_y_range = [iy for iy in range(click_y - 10, click_y + 10)]
                distance = int(dist([coord1x, coord1y], [coord2x, coord2y]))

                # checking along the line if the click is witnin +- 10 around the click
                for dist_mult in range(1, distance - 1):
                    dist_mult = dist_mult / distance
                    x_check = int(coord1x) + int(round(vector[0] * dist_mult, 0))
                    y_check = int(coord1y) + int(round(vector[1] * dist_mult, 0))
                    if x_check in click_x_range and y_check in click_y_range:
                        self.line_is_active = True
                        if self._add_to_lines:
                            self._active_lines.append(key)
                        elif self._add_to_lines== False:
                            if key in self._active_lines:
                                self._active_lines.remove(key)
                        self._main_canvas.delete('all')
                        break
        self.draw_canvas()

    def right_click(self,event):
        '''
        Event when right click.
        :param evnet:
        :return:
        '''
        self._previous_drag_mouse = [event.x, event.y]
        self._active_lines = []
        self._main_canvas.delete('all')
        self.draw_canvas()

    def left_click_load_box(self, *event):
        '''
        Load boxes consist of self._load_obj_box (active/non-active) and self._load_obj_lines_box (listing assosiated
         lines). Both is tkinter ListBox objects.
        :param events:
        :return:
        '''
        self._load_obj_lines_box.delete(0,'end')
        self._active_lines = []

        if len(self._load_objects)!=0:
            self._canvas_properties.delete('all')
            current_selection = self._load_obj_box.get('active')
            current_object = self._load_objects[current_selection][0]
            current_lines = self._load_objects[current_selection][1]
            self._new_assisiate_load.set(current_selection)

            # drawing properties in the canvas
            self._canvas_properties.create_text([140, 100], text=self._load_objects[current_selection][0])

            for line in sorted([get_num(line) for line in current_lines]):
                self._load_obj_lines_box.insert('end','line'+str(line))
                self._active_lines.append('line'+str(line))

            if current_object.is_static():
                self._new_static_load_name.set(current_object.get_load_parmeters()[8])
                self._new_static_draft.set(current_object.get_load_parmeters()[7])
                self._new_static_condition.set(current_object.get_load_parmeters()[4])
            else:
                self._new_dynamic_load_name.set(current_object.get_load_parmeters()[8])
                self._new_load_poly_third.set(current_object.get_load_parmeters()[0])
                self._new_load_poly_second.set(current_object.get_load_parmeters()[1])
                self._new_load_poly_first.set(current_object.get_load_parmeters()[2])
                self._new_load_poly_const.set(current_object.get_load_parmeters()[3])
                self._new_load_manual_pressure.set(current_object.get_load_parmeters()[6])
                self._new_dyn_load_condition.set(current_object.get_load_parmeters()[4])
                self._new_limit_state.set(current_object.get_load_parmeters()[9])

            if current_object.get_load_parmeters()[4] == 'slamming':
                self._new_slamming_pressure.set(current_object.get_load_parmeters()[3])
                self._new_slamming_pressure_name.set(current_object.get_load_parmeters()[8])
                self._new_slamming_pl_mult.set(current_object.get_load_parmeters()[10])
                self._new_slamming_stf_mult.set(current_object.get_load_parmeters()[11])

            self._load_obj_box.update()
            self._ent_assosiate_load.update_idletasks()
            self.draw_canvas(load_selected=True)

    def import_update(self):

        for load, data in self._load_objects.items():
            self._load_obj_box.insert('end', load)
            self._load_options.append(load)

        self._ent_assosiate_load.destroy()
        self._ent_assosiate_load = tk.OptionMenu(self._frame, self._new_assisiate_load, *tuple(self._load_options))
        self._ent_assosiate_load.place(relx=0.85,rely=0.21)

if __name__ == '__main__':
    root = tk.Tk()
    my_app = CreateLoadWindow(master=root)
    root.mainloop()